#include <cstdio>
#include <iostream>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>
//#include <argp.h>
#include <ctime>   // for calculation of computation time
#include <list>
//#include "config.h"
using namespace std;	// to make complex template visible

//#include "InOut.h"
#include "Divisions.h"
//#include "Helpers.h"


typedef list<float> float_list;
enum complex_component {cc_real, cc_imag};

class signal_fkt;

/* The transform should require only time and space that is necessary for the special cases
      real wavelet on    real signal
	complex wavelet on    real signal
	   real wavelet on complex signal
	complex wavelet on complex signal
	
	That's why the transform is split in following parts:
		the transform is started from the wavelet class,
			it decides whether to use real wavelet, complex wavelet or fast bspline wavelet
		then the control is handed over to signal class
			which in turn decides about transform real or complex signal
		post-processing: in the first three cases the result is already complete at this point,
			only in the complex-complex case we have data sets of four separate products
			(real and complex part of wavelet with real and complex part of the signal)
			which we have to add or sub now
*/

/*
 *	WAVELET FUNKTIONEN
 */
class wavelet_fkt 
{
public:
	float cpsi,lsupp,rsupp;
	char namewave[15];

	virtual void init() = 0;
	virtual void reset() {};
	virtual void init_args(const char *arguments);
	virtual float evaluate(float x) const = 0;
	float evaluate_clip(float x) const;
	virtual void transform(const signal_fkt *sf, division_range *adiv,  const char *outputbasename, 
													const arguments &arguments, CArray<FLOAT,FLOAT>* pWaveletCoeffizients) = 0;
	virtual void discretize(float *w, long lpos, long rpos, float xc, float yc) const;

	void get_supp (float as, long &lpos, long &rpos) const;

	void get_max_supp (const arguments &arguments, long &sparebegin, long &spareend) const;
};

// Unterklasse Reelle	WAVELET FUNKTIONEN

class wavelet_real:public wavelet_fkt {
	virtual void transform(const signal_fkt *sf, division_range *adiv, const char *outputbasename, 
													const arguments &arguments, CArray<FLOAT,FLOAT>* pWaveletCoeffizients);
};

// Reelle Waveletfunktionen
class wavelet_haar:public wavelet_real {
	virtual void init();
	virtual float evaluate(float x) const;
};

class wavelet_mexicanhat:public wavelet_real {
	virtual void init();
	virtual float evaluate(float x) const;
};

class wavelet_sinus:public wavelet_real {
	virtual void init();
	virtual float evaluate(float x) const;
};

class wavelet_poisson:public wavelet_real {
	virtual void init();
	virtual float evaluate(float x) const;
};


// Unterklasse Komplexe	WAVELET FUNKTIONEN

class wavelet_complex:public wavelet_fkt {
public:
//	virtual void reset();
protected:
	complex_component cc;
	virtual void transform(const signal_fkt *sf, division_range *adiv, const char *outputbasename,
													const arguments &arguments, CArray<FLOAT,FLOAT>* pWaveletCoeffizients);

	void base_transform(const signal_fkt *sf, division_range *adiv, const char *outputbasename, 
											const arguments &arguments, CArray<FLOAT,FLOAT>* pWaveletCoeffizientsReal,
											CArray<FLOAT,FLOAT>* pWaveletCoeffizientsImg,
											CArray<FLOAT,FLOAT>* pWaveletCoeffizients);
};

// Komplexe Waveletfunktionen
class wavelet_morlet:public wavelet_complex {
	float scale;
	virtual void init();
	virtual void init_args(const char *arguments);
	virtual float evaluate(float x) const;
};

class wavelet_cauchy:public wavelet_complex {
	float freq, order;
	virtual void init();
	virtual void init_args(const char *arguments);
	virtual float evaluate(float x) const ;
};

// cauchy wavelet where you can specify the variation in scale and time direction
class wavelet_spinning:public wavelet_complex {
	float freq;
	complex<float> alpha;

	virtual void init() ;
	virtual void init_args(const char *arguments);
	virtual float evaluate(float x) const;
};

class wavelet_bspline:public wavelet_complex {
public:
	float_list* pwidths;
//	CList<FLOAT,FLOAT> widths;
	virtual void init();
	void reset();
	virtual void init_args(const char *arguments);
	virtual float evaluate(float x) const ;
	// similar to the global 'smooth' routine
	// buffer z must contain at least len+n-1 words
	void wide_smooth(float *z, const float *ya, long len, long n) const ;
	virtual void discretize(float *w, long lpos, long rpos, float xc, float yc) const;
	// Swapping of 'const' buffers can't be used for non-constant buffers - why says gcc that?
	inline void swap_buffer(float *&a, float *&b) ;
	inline void swap_buffer(const float *&a, const float *&b);

	void transform(const signal_fkt *sf, division_range *adiv, const char *outputbasename,
													const arguments &arguments,	CArray<FLOAT,FLOAT>* pWaveletCoeffizients);
	void fasttransform(const signal_fkt *sf, division_range *adiv, const char *outputbasename,
											const arguments &arguments, 
											CArray<FLOAT,FLOAT>* pWaveletCoeffizientsReal,
											CArray<FLOAT,FLOAT>* pWaveletCoeffizientsImg,
											CArray<FLOAT,FLOAT>* pWaveletCoeffizients);

};





/****************** signal classes **************************/

/* We have to perform different actions depending on whether the signal is real or complex
   and if the signal is discretized or continuous.
	We can't put these two aspects easily in a class structure,
	thus we will use sub classes for distinguishing between
	discretized and continuous signals. The decision between real and complex values
	has to be made by each routine separately.
*/


enum superimpose_modes {add, sub};

class signal_fkt {
public:
	float *fr,      *fi,  // pointer to zeroth value
	       *fralloc, *fialloc;  // pointer to the start of the allocated memory block
	long totalbegin, totalend;


	bool complex_valued;

public:
	signal_fkt() : complex_valued(false) {fialloc=NULL;fralloc=NULL;};
	~signal_fkt();
	virtual float evaluate_real(float x) const = 0;
	virtual float evaluate_imag(float x) const;
	virtual const float *discretize(const arguments &args, const wavelet_fkt *wf, complex_component cc) const = 0;
	virtual void free_discretized(const float *f) const = 0;
	inline bool is_complex() const {return complex_valued;};
	void transform(const wavelet_fkt *wf, division_range *adiv, const char *outputbasename, 
								const arguments &arguments,
								CArray<FLOAT,FLOAT>* pWaveletCoeffizientsReal,
								CArray<FLOAT,FLOAT>* pWaveletCoeffizientsImg,
								CArray<FLOAT,FLOAT>* pWaveletCoeffizients) const;
	void merge_complex (const wavelet_complex *wf, const arguments &arguments, const char *outputbasename, 
											const char *outext, const char *inaext, const char *inbext, superimpose_modes mode,
											CArray<FLOAT,FLOAT>* pWaveletCoeffizients) const;
	void virtual write (FILE *file, const arguments &args, complex_component cc) const = 0;
};

class signal_continuous:public signal_fkt 
{
	virtual const float *discretize(const arguments &args, const wavelet_fkt *wf, complex_component cc) const;
	virtual void free_discretized(float *f) const;
	void virtual write (FILE *file, const arguments &args, complex_component cc) const;
};


class signal_discrete:public signal_fkt 
{

public:
	/*
	  ... 0 0 0 0 0 | data data data data data | 0 0 0 0 0 ...     - in our imagination, the data stream consisting of the real data is extended with zeros before the beginning and at after the end
	        | <-sparebegin- |         | -spareend-> |
	                      bmin      bmax
	*/
	signal_discrete(SAMPLE* input,INT input_length, const arguments &args, const wavelet_fkt *wf);
	virtual const float *discretize(const arguments &args, const wavelet_fkt *wf, complex_component cc) const;
	virtual void free_discretized(const float *f) const {};
	virtual float evaluate_real(float x) const;
	virtual float evaluate_imag(float x) const;
	virtual void write (FILE *file, const arguments &args, complex_component cc) const;
};




struct wavelet_desc {
	char *id, *help;
};

static const struct wavelet_desc wavelet_desc_arr[] = {
#include "wavelet_array.h"
	{ 0 }
};

struct signal_desc {
	char *id, *help;
};

static const struct signal_desc signal_desc_arr[] = {
#include "signal_array.h"
	{ 0 }
};
