//
// MsHelp.h : Help Services for HTML Help 1.0
// Copyright (C) 1999-2004 MGTEK. All rights reserved.
//

#if _MSC_VER > 1000
#pragma once
#endif

#ifndef __MSHELP_H_INCLUDED__
#define __MSHELP_H_INCLUDED__

#ifndef MSHELP_USE_STRSAFE
#if defined(_STRSAFE_H_INCLUDED_) || defined(__GOT_SECURE_LIB__)
#define MSHELP_USE_STRSAFE
#endif
#endif

/////////////////////////////////////////////////////////////////////////////

class CMsHelp
{
	typedef HWND (WINAPI * FHTMLHELP)(HWND hWnd, LPCTSTR pszFile, UINT nCommand, DWORD_PTR dwData);

public:
	struct QUERY
	{
		int			cbStruct;
		BOOL		fUniCodeStrings;
		LPCTSTR		pszSearchQuery;
		LONG		iProximity;
		BOOL		fStemmedSearch;
		BOOL		fTitleOnly;
		BOOL		fExecute;
		LPCTSTR		pszWindow;
	};

	struct KEYWORD
	{
		int			cbStruct;
		BOOL		fReserved;
		LPCTSTR		pszKeywords;
		LPCTSTR		pszUrl;
		LPCTSTR		pszMsgText;
		LPCTSTR		pszMsgTitle;
		LPCTSTR		pszWindow;
		BOOL		fIndexOnFail;
	};

	struct ALINK
	{
		int			cbStruct;
		BOOL		fReserved;
		LPCSTR		pszKeywords;
		LPCSTR		pszUrl;
		LPCSTR		pszMsgText;
		LPCSTR		pszMsgTitle;
		LPCSTR		pszWindow;
		BOOL		fIndexOnFail;
	};

	struct POPUP
	{
		int			cbStruct;
		HINSTANCE	hinst;
		UINT		idString;
		LPCTSTR		pszText;
		POINT		pt;
		COLORREF	clrForeground;
		COLORREF	clrBackground;
		RECT		rcMargins;
		LPCTSTR		pszFont;
	};

	struct LAST_ERROR
	{
		int			cbStruct;
		HRESULT		hr;
		BSTR		description;
	};

// Construction
public:
	CMsHelp(LPCTSTR pszFileName = NULL) : m_hWnd(NULL)
	{
		m_acFileName[0] = _T('\0');
		SetHelpFile(pszFileName);
	}

// Attributes
public:
	HWND	m_hWnd;
	TCHAR	m_acFileName[MAX_PATH];

// Operations
public:
	// Set the parent window to use for help.
	void SetWindow(HWND hWnd = ::GetDesktopWindow())
	{
		ATLASSERT(hWnd != NULL);
		m_hWnd = hWnd;
	}

	// Set the help file to use for help.
	HRESULT SetHelpFile(LPCTSTR pszFileName)
	{
		if (pszFileName != NULL)
		{
			// Get the full pathname to become independent of the current directory.
			LPTSTR lpFilePart;
			TCHAR acFullPath[MAX_PATH];
			UINT nChars = ::GetFullPathName(pszFileName, MAX_PATH, acFullPath, &lpFilePart);
			if (nChars == 0)
				return HRESULT_FROM_WIN32(::GetLastError());
			if (nChars > MAX_PATH)
				return HRESULT_FROM_WIN32(ERROR_INSUFFICIENT_BUFFER);

			// Get the short path to the help file to work around Unicode bugs in 'hhctrl.ocx'.
			nChars = ::GetShortPathName(acFullPath, m_acFileName, MAX_PATH);
			if (nChars == 0)
				return HRESULT_FROM_WIN32(::GetLastError());
			if (nChars > MAX_PATH)
				return HRESULT_FROM_WIN32(ERROR_INSUFFICIENT_BUFFER);
		}
		else
		{
			m_acFileName[0] = _T('\0');
		}
		return S_OK;
	}

	// Display the table of contents tab in the navigation pane.
	HRESULT DisplayTocTab()
	{
		if (m_acFileName[0] == _T('\0'))
			return HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND);
		return DisplayTocTab(m_hWnd, m_acFileName);
	}

	// Display the index tab in the navigation pane.
	HRESULT DisplayIndexTab()
	{
		if (m_acFileName[0] == _T('\0'))
			return HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND);
		return DisplayIndexTab(m_hWnd, m_acFileName);
	}

	// Display the search tab in the navigation pane.
	HRESULT DisplaySearchTab()
	{
		if (m_acFileName[0] == _T('\0'))
			return HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND);
		QUERY query = { sizeof(QUERY) };
		return DisplaySearchTab(m_hWnd, m_acFileName, &query);
	}

	// Display topic page.
	HRESULT DisplayTopic(LPCTSTR pszTopic)
	{
		ATLASSERT(pszTopic != NULL);
		if (m_acFileName[0] == _T('\0'))
			return HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND);
		return DisplayTopic(m_hWnd, m_acFileName, pszTopic);
	}

	// Looks up a topic based on a keyword.
	HRESULT LookupKeyword(LPCTSTR pszKeywords)
	{
		ATLASSERT(pszKeywords != NULL);
		if (m_acFileName[0] == _T('\0'))
			return HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND);
		KEYWORD keyword = { sizeof(KEYWORD) };
		keyword.pszKeywords = pszKeywords;
		return LookupKeyword(m_hWnd, m_acFileName, &keyword);
	}

	// Looks up a topic based on a A-link keyword.
	HRESULT LookupALink(LPCTSTR pszKeywords)
	{
		ATLASSERT(pszKeywords != NULL);
		if (m_acFileName[0] == _T('\0'))
			return HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND);
		ALINK alink = { sizeof(ALINK) };
#ifdef _UNICODE
		int nSize = ::WideCharToMultiByte(CP_ACP, 0, pszKeywords, -1, NULL, 0, NULL, NULL);
		LPSTR psz = (LPSTR)_alloca(nSize);
		::WideCharToMultiByte(CP_ACP, 0, pszKeywords, -1, psz, nSize, NULL, NULL);
		alink.pszKeywords = psz;
#else
		alink.pszKeywords = pszKeywords;
#endif
		return LookupALink(m_hWnd, m_acFileName, &alink);
	}

	// Display context sensitive help.
	HRESULT DisplayContextHelp(UINT nIDTopic)
	{
		if (m_acFileName[0] == _T('\0'))
			return HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND);
		return DisplayContextHelp(m_hWnd, m_acFileName, nIDTopic);
	}

	// Display a simple text popup window.
	// The text definitions for the popups must be located in the file
	// 'popups.txt' in the root folder of the chm file.
	HRESULT DisplayTextPopup(UINT nTopicId, POINT pt)
	{
		static LPCTSTR pszPopups = _T("::/popups.txt");

		if (m_acFileName[0] == _T('\0'))
			return HRESULT_FROM_WIN32(ERROR_FILE_NOT_FOUND);

		size_t nChars = _tcslen(m_acFileName) + _tcslen(pszPopups) + 1;
		LPTSTR pszFileName = static_cast<LPTSTR>(_alloca(nChars * sizeof(TCHAR)));
#ifdef MSHELP_USE_STRSAFE
		_tcscpy_s(pszFileName, nChars, m_acFileName);
		_tcscat_s(pszFileName, nChars, pszPopups);
#else
		_tcscpy(pszFileName, m_acFileName);
		_tcscat(pszFileName, pszPopups);
#endif
		POPUP popup = { 0 };
		RECT rcDefault = { -1, -1, -1, -1 };
		popup.cbStruct = sizeof(POPUP);
		popup.idString = nTopicId;
		popup.pt = pt;
		popup.clrForeground = static_cast<COLORREF>(-1);
		popup.clrBackground = static_cast<COLORREF>(-1);
		popup.rcMargins = rcDefault;
		popup.pszFont = _T("");
		return DisplayTextPopup(m_hWnd, pszFileName, &popup);
	}

	/// Low-level HtmlHelp API.

	// Displays a specific topic: file.chm::/topic.html>window
	static HRESULT DisplayTopic(HWND hWnd, LPCTSTR pszFileName, LPCTSTR pszTopic = NULL)
	{
		return HtmlHelpSafeApi(hWnd, pszFileName, 0x0000, reinterpret_cast<DWORD_PTR>(pszTopic));
	}

	// Display the table of contents tab in the navigation pane.
	static HRESULT DisplayTocTab(HWND hWnd, LPCTSTR pszFileName, LPCTSTR pszTopic = NULL)
	{
		return HtmlHelpSafeApi(hWnd, pszFileName, 0x0001, reinterpret_cast<DWORD_PTR>(pszTopic));
	}

	// Display the index tab in the navigation pane.
	static HRESULT DisplayIndexTab(HWND hWnd, LPCTSTR pszFileName, LPCTSTR pszKeyword = NULL)
	{
		return HtmlHelpSafeApi(hWnd, pszFileName, 0x0002, reinterpret_cast<DWORD_PTR>(pszKeyword));
	}

	// Display the search tab in the navigation pane.
	// Functionality is partially broken (see KB 241381).
	static HRESULT DisplaySearchTab(HWND hWnd, LPCTSTR pszFileName, QUERY* pQuery)
	{
		return HtmlHelpSafeApi(hWnd, pszFileName, 0x0003, reinterpret_cast<DWORD_PTR>(pQuery));
	}

	// Locates and selects the contents entry for the help topic.
	static HRESULT Sync(HWND hWnd, LPCTSTR pszFileName, LPCTSTR pszTopic)
	{
		return HtmlHelpSafeApi(hWnd, pszFileName, 0x0009, reinterpret_cast<DWORD_PTR>(pszTopic));
	}

	// Looks up a topic based on a keyword.
	static HRESULT LookupKeyword(HWND hWnd, LPCTSTR pszFileName, KEYWORD* pKeywords)
	{
		return HtmlHelpSafeApi(hWnd, pszFileName, 0x000D, reinterpret_cast<DWORD_PTR>(pKeywords));
	}

	// Looks up a topic based on a associative keyword.
	static HRESULT LookupALink(HWND hWnd, LPCTSTR pszFileName, ALINK* pKeywords)
	{
		return HtmlHelpSafeApi(hWnd, pszFileName, 0x0013, reinterpret_cast<DWORD_PTR>(pKeywords));
	}

	// Display context sensitive help.
	static HRESULT DisplayContextHelp(HWND hWnd, LPCTSTR pszFileName, UINT nIDTopic)
	{
		return HtmlHelpSafeApi(hWnd, pszFileName, 0x000F, nIDTopic);
	}

	// Display a help topic in a pop-up window.
	static HRESULT DisplayTextPopup(HWND hWnd, LPCTSTR pszFileName, POPUP* pPopup)
	{
		return HtmlHelpSafeApi(hWnd, pszFileName, 0x000E, reinterpret_cast<DWORD_PTR>(pPopup));
	}

	// Closes all windows opened by the calling program.
	static HRESULT CloseAll()
	{
		return HtmlHelpApi(NULL, NULL, 0x0012, 0);
	}

	// return the last error cause by HtmlHelp.
	static HRESULT GetLastError()
	{
		LAST_ERROR error = { sizeof(LAST_ERROR) };
		HRESULT hr = HtmlHelpApi(NULL, NULL, 0x0014, reinterpret_cast<DWORD_PTR>(&error));
		if (FAILED(hr))
			return hr;
		if (error.description != NULL)
			::SysFreeString(error.description);
		if (FAILED(error.hr))
			return error.hr;
		return S_OK;
	}

	// A wrapper for HtmlHelp function in 'hhctrl.ocx'.
	static HRESULT HtmlHelpApi(HWND hWnd, LPCTSTR pszFileName, UINT nCommand, DWORD_PTR dwData, HWND* phWndResult = NULL)
	{
		if (m_pHtmlHelp == NULL)
		{
			HRESULT hr = LoadHtmlHelpControl();
			if (FAILED(hr))
				return hr;
		}
		HWND hWndResult = reinterpret_cast<FHTMLHELP>(m_pHtmlHelp)(hWnd, pszFileName, nCommand, dwData);
		if (phWndResult != NULL)
			*phWndResult = hWndResult;
		return S_OK;
	}

	// Same as HtmlHelpApi but with results check.
	static HRESULT HtmlHelpSafeApi(HWND hWnd, LPCTSTR pszFileName, UINT nCommand, DWORD_PTR dwData)
	{
		HWND hWndResult = NULL;
		HRESULT hr = HtmlHelpApi(hWnd, pszFileName, nCommand, dwData, &hWndResult);
		if (FAILED(hr))
			return hr;
		if (hWndResult == NULL)
			return GetLastError();
		return S_OK;
	}

private:
	// Load the HTML Help Active-X control.
	static HRESULT LoadHtmlHelpControl()
	{
		static LPCTSTR pszLocation = _T("CLSID\\{ADB880A6-D8FF-11CF-9377-00AA003B7A11}\\InprocServer32");

		HMODULE hModule = NULL;

		// Try to load 'hhctrl.ocx' at it's registered location.
		HKEY hKey = NULL;
		HRESULT hr = ::RegOpenKeyEx(HKEY_CLASSES_ROOT, pszLocation, 0, KEY_READ, &hKey);
		if (SUCCEEDED(hr))
		{
			TCHAR acPath[MAX_PATH];
			DWORD dwType = 0;
			DWORD dwSize = MAX_PATH;
			hr = ::RegQueryValueEx(hKey, NULL, NULL, &dwType, reinterpret_cast<LPBYTE>(acPath), &dwSize);
			::RegCloseKey(hKey);
			if (SUCCEEDED(hr) && dwType == REG_SZ)
				hModule = ::LoadLibrary(acPath);
		}

		if (hModule == NULL)
		{
			// Try to load 'hhctrl.ocx' from it's default path.
			hModule = ::LoadLibrary(_T("hhctrl.ocx"));
			if (hModule == NULL)
			{
				// Failed to load library.
				return HRESULT_FROM_WIN32(::GetLastError());
			}
		}

		// Locate HtmlHelp function.
#ifdef UNICODE
		FARPROC proc = ::GetProcAddress(hModule, (LPCSTR)0x0F);		// HtmlHelpW
#else
		FARPROC proc = ::GetProcAddress(hModule, (LPCSTR)0x0E);		// HtmlHelpA
#endif
		if (proc == NULL)
		{
			// Failed to locate HtmlHelp function.
			DWORD dwError = ::GetLastError();
			::FreeLibrary(hModule);
			return HRESULT_FROM_WIN32(dwError);
		}

		// Success.
		m_hModule = hModule;
		m_pHtmlHelp = proc;
		return S_OK;
	}

private:
	static HMODULE	m_hModule;		// Module handle to 'hhctrl.ocx'.
	static FARPROC	m_pHtmlHelp;	// HtmlHelp function in 'hhctrl.ocx'.
};

/////////////////////////////////////////////////////////////////////////////

__declspec(selectany) HMODULE CMsHelp::m_hModule = NULL;
__declspec(selectany) FARPROC CMsHelp::m_pHtmlHelp = NULL;

/////////////////////////////////////////////////////////////////////////////

#endif // __MSHELP_H_INCLUDED__
