// RekoView.cpp: Implementierungsdatei
//

#include "stdafx.h"
#include "Sounds.h"
#include "RekoView.h"
#include "GrafikDeviceDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MAX(a,b)        ( (a) > (b) ? (a) : (b) )

/////////////////////////////////////////////////////////////////////////////
// CRekoView

IMPLEMENT_DYNCREATE(CRekoView, CView)

CRekoView::CRekoView()
{
	INT Items=AfxGetMainWnd()->GetMenu()->GetMenuItemCount();
	AfxGetMainWnd()->GetMenu()->GetSubMenu(Items>5?5:3)->AppendMenu(MF_STRING|MF_ENABLED,ID_FENSTER_REKO,"3D-Rekonstruktion");
	AfxGetMainWnd()->DrawMenuBar();
}

void CRekoView::IniFileRead()
{
	//INIFile Werte lesen
	m_Tau						= *theApp.GetIniRekoTau();
	m_clrBackBuffer	=	*theApp.GetIniRekoclrBackBuffer();
	m_SizePts				= *theApp.GetIniRekoSizePts();
	m_bIsPunkte			= *theApp.GetIniRekobIsPunkte();
	m_NumRekoPoints = *theApp.GetIniRekoNumRekoPoints();
	m_bIsBunt				= *theApp.GetIniRekobIsBunt();
}

void CRekoView::IniFileWrite()
{
	//INIFile Werte schreiben
	*theApp.GetIniRekoTau()=m_Tau;
	*theApp.GetIniRekoclrBackBuffer()=m_clrBackBuffer;
	*theApp.GetIniRekoSizePts()=m_SizePts*2;
	*theApp.GetIniRekobIsPunkte()=m_bIsPunkte;
	*theApp.GetIniRekoNumRekoPoints()=m_NumRekoPoints;
	*theApp.GetIniRekobIsBunt()=m_bIsBunt;
}

CRekoView::~CRekoView()
{
	IniFileWrite();		
	INT Items=AfxGetMainWnd()->GetMenu()->GetMenuItemCount();
	AfxGetMainWnd()->GetMenu()->GetSubMenu(Items>5?5:3)->RemoveMenu(ID_FENSTER_REKO,MF_BYCOMMAND);
	AfxGetMainWnd()->DrawMenuBar();
	CSoundsDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	pDoc->m_pViewD3DReko=NULL;
	Cleanup3DEnvironment();
}


BEGIN_MESSAGE_MAP(CRekoView, CView)
	//{{AFX_MSG_MAP(CRekoView)
	ON_WM_MOUSEMOVE()
	ON_WM_SIZE()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_COMMAND(IDM_TOGGLEHELP, OnTogglehelp)
	ON_COMMAND(IDM_TOGGLESTART, OnTogglestart)
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(IDM_SINGLESTEP, OnSinglestep)
	ON_COMMAND(IDM_CHANGEDEVICE, OnChangedevice)
	ON_WM_MOUSEWHEEL()
	ON_EN_CHANGE(IDC_PUNKTE_FELD, OnChangePunkteFeld)
	ON_EN_CHANGE(IDC_TAU, OnChangeTau)
	ON_BN_CLICKED(IDC_PUNKTE, OnPunkte)
	ON_BN_CLICKED(IDC_LINIEN, OnLinien)
	ON_BN_CLICKED(IDC_FESTEFARBE, OnFestefarbe)
	ON_BN_CLICKED(IDC_FARBWAHL, OnFarbwahl)
	ON_BN_CLICKED(IDC_BUTTON_DREHZR, OnButtonDrehzr)
	ON_BN_CLICKED(IDC_BUTTON_DREHZL, OnButtonDrehzl)
	ON_BN_CLICKED(IDC_BUTTON_DREHYR, OnButtonDrehyr)
	ON_BN_CLICKED(IDC_BUTTON_DREHYL, OnButtonDrehyl)
	ON_BN_CLICKED(IDC_BUTTON_DREHXR, OnButtonDrehxr)
	ON_BN_CLICKED(IDC_BUTTON_DREHXL, OnButtonDrehxl)
	ON_BN_CLICKED(IDC_BUNT, OnBunt)
	ON_BN_CLICKED(IDC_RENDERBTN, OnRenderbtn)
	ON_BN_CLICKED(IDC_RESETVIEW, OnResetview)
	ON_COMMAND(ID_SAVE, OnSave)
	ON_COMMAND(ID_EDIT_CLEARALL, OnEditClearall)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_BN_CLICKED(IDC_BKGCLR, OnBkgclr)
	ON_WM_HELPINFO()
	ON_EN_CHANGE(IDC_NUMPUNKTE_FELD, OnChangeNumpunkteFeld)
	ON_WM_CREATE()
	ON_COMMAND(ID_EXPORTDATA, OnExportdata)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Zeichnung CRekoView 

void CRekoView::OnDraw(CDC* pDC)
{
  // Handle paint messages when the app is not ready
  if( m_pd3dDevice)// && !m_bReady )
    {
    if( m_bWindowed )
			{
				Render3DEnvironment();
      }
    }
}

/////////////////////////////////////////////////////////////////////////////
// Diagnose CRekoView

#ifdef _DEBUG
void CRekoView::AssertValid() const
{
	CView::AssertValid();
}

void CRekoView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// Behandlungsroutinen fr Nachrichten CRekoView 

void CRekoView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();

	if (*theApp.GetIniAllgemeinGrundschulprofil())
	{
		CSoundsDoc* pDoc=GetDocument();
		*pDoc->GetFFTPoints()=8192;
		pDoc->UpdateStatusBar();
	}

		CString Text;
		Text.LoadString(IDS_REKOTITLE);
		m_strWindowTitle    = Text.GetBuffer(Text.GetLength());
    m_bUseDepthBuffer   = TRUE;

    m_pFont            = new CD3DFont( _T("Century Gothic"), 12, 0);//D3DFONT_BOLD );
    m_pFontSmall       = new CD3DFont( _T("Century Gothic"),  9, 0);//D3DFONT_BOLD );
		m_pVBZeitreihe		 = NULL;
		m_pVBKoord				 = NULL;
    m_dwNumVertices    = 0;

    ZeroMemory( m_bKey, sizeof(m_bKey) );
    m_fSpeed           = 5.0f;
    m_fAngularSpeed    = 1.0f;
    m_bShowHelp        = FALSE;

    m_vVelocity        = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
    m_vAngularVelocity = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );

		x							=	0.0f;
		y							= 0.0f;
		z							=	0.0f;

		m_RekoPtArray.Add(0);

		m_Recalc			=	FALSE;
		
		m_clrRekoREF=RGB(255,0,0);
		m_clrText=RGB(255,255,255);
		m_clrBkg=RGB(0,0,0);

	IniFileRead();
	m_pParentRekoFrame=(CD3DRekoFrame*)GetParentFrame();
	m_pTauCtrl=(CSpinButtonCtrl*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_SPINTAU);
	m_pTauCtrl->SetRange(1,500);

  m_pTauCtrl->SetBuddy(m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_TAU));
	m_pTauCtrl->SetPos(m_Tau);

//	CSpinButtonCtrl* pNumPunkteCtrl=(CSpinButtonCtrl*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_NUMPUNKTE_SPIN);
//	pNumPunkteCtrl->SetRange(1,10000);
//	pNumPunkteCtrl->SetBuddy(m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_NUMPUNKTE_FELD));
//	CEdit* pNumPunkteCtrl=(CEdit*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_NUMPUNKTE_FELD);
	m_pParentRekoFrame->m_wndDlgBar.SetDlgItemInt(IDC_NUMPUNKTE_FELD,m_NumRekoPoints);

	m_pPunkteCtrl=(CSpinButtonCtrl*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_PUNKTE_SPIN);
	m_pPunkteCtrl->SetRange(1,20);
	m_pPunkteCtrl->SetBuddy(m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_PUNKTE_FELD));
	m_pPunkteCtrl->SetPos((INT)m_SizePts);

	m_pTauWertCtrl=(CEdit*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_SPINTAU);

	m_pPunkteWertCtrl=(CEdit*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_PUNKTE_FELD);;
	if (m_bIsPunkte) 
		m_pPunkteWertCtrl->EnableWindow(TRUE);
	else
		m_pPunkteWertCtrl->EnableWindow(FALSE);

	m_pRadioFesteFarbe=(CButton*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_FESTEFARBE);
	m_pRadioBunt=(CButton*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_BUNT);
	m_FarbButton.SubclassDlgItem(IDC_FARBWAHL, &m_pParentRekoFrame->m_wndDlgBar);
	if (m_bIsBunt) 
	{
		m_FarbButton.EnableWindow(FALSE);
		m_FarbButton.SetColour(GetSysColor(COLOR_INACTIVECAPTION),GetSysColor(COLOR_MENU));
		m_pRadioBunt->SetCheck(1);
		m_pRadioFesteFarbe->SetCheck(0);
	}
	else
	{
		m_FarbButton.EnableWindow(TRUE);
		m_FarbButton.SetColour(m_clrText,m_clrRekoREF);
		m_pRadioBunt->SetCheck(0);
		m_pRadioFesteFarbe->SetCheck(1);
	}

	m_pRadioLinien=(CButton*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_LINIEN);
	m_pRadioPunkte=(CButton*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_PUNKTE);
	if (m_bIsPunkte) 
	{
		m_pRadioLinien->SetCheck(0);
		m_pRadioPunkte->SetCheck(1);
	}
	else
	{
		m_pRadioLinien->SetCheck(1);
		m_pRadioPunkte->SetCheck(0);
	}



	m_clrReko=D3DCOLOR_XRGB(GetRValue(m_clrRekoREF),\
													GetGValue(m_clrRekoREF),\
													GetBValue(m_clrRekoREF));


	m_FarbButtonBkg.SubclassDlgItem(IDC_BKGCLR, &m_pParentRekoFrame->m_wndDlgBar);
	m_FarbButtonBkg.EnableWindow(TRUE);
	m_FarbButtonBkg.SetColour(m_clrText,m_clrBkg);

  m_hwndRenderWindow = GetSafeHwnd();
	CD3DApplication::m_hWnd = m_hwndRenderWindow;
  HRESULT hr=CD3DApplication::Create( AfxGetInstanceHandle() );
	if (hr==D3DAPPERR)	//Fehler bei der Erstellung des Direct3DApplicationObjekts
	{
		CSoundsDoc* pDoc=GetDocument();
		pDoc->m_pViewD3DReko=(CD3DRekoFrame*)this->GetParentFrame();
		m_bReady=FALSE;
		pDoc->m_bIsNot3D=TRUE;
	}
	else
	{
		//Farbpalette erstellen
		CSoundsDoc* pDoc=GetDocument();
		CDC* pDC=GetDC();
		pDC->SelectPalette(pDoc->GetPalette(), FALSE);
		pDC->RealizePalette();
	}
	if (*theApp.GetIniAllgemeinGrundschulprofil()) 
	{
		SwitchDlgItems(FALSE);
	}
	else
	{
		SwitchDlgItems(TRUE);
	}
}


void CRekoView::SwitchDlgItems(BOOL bVisible)
{
	m_pParentRekoFrame=(CD3DRekoFrame*)GetParentFrame();

	if (bVisible) 
	{
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_STATIC_TAU)->ShowWindow(SW_SHOW);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_STATIC_DREHUNG)->ShowWindow(SW_SHOW);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_STATIC_X)->ShowWindow(SW_SHOW);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_STATIC_Y)->ShowWindow(SW_SHOW);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_STATIC_Z)->ShowWindow(SW_SHOW);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_STATIC_POINTS)->ShowWindow(SW_SHOW);

		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_TAU)->ShowWindow(SW_SHOW);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_BUTTON_DREHXL)->ShowWindow(SW_SHOW);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_BUTTON_DREHXR)->ShowWindow(SW_SHOW);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_BUTTON_DREHYL)->ShowWindow(SW_SHOW);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_BUTTON_DREHYR)->ShowWindow(SW_SHOW);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_BUTTON_DREHZL)->ShowWindow(SW_SHOW);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_BUTTON_DREHZR)->ShowWindow(SW_SHOW);
	
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_SPINTAU)->ShowWindow(SW_SHOW);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_NUMPUNKTE_FELD)->ShowWindow(SW_SHOW);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_RESETVIEW)->ShowWindow(SW_SHOW);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(ID_SAVE)->ShowWindow(SW_SHOW);
	}
	else
	{
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_STATIC_TAU)->ShowWindow(SW_HIDE);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_STATIC_DREHUNG)->ShowWindow(SW_HIDE);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_STATIC_X)->ShowWindow(SW_HIDE);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_STATIC_Y)->ShowWindow(SW_HIDE);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_STATIC_Z)->ShowWindow(SW_HIDE);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_STATIC_POINTS)->ShowWindow(SW_HIDE);

		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_TAU)->ShowWindow(SW_HIDE);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_BUTTON_DREHXL)->ShowWindow(SW_HIDE);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_BUTTON_DREHXR)->ShowWindow(SW_HIDE);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_BUTTON_DREHYL)->ShowWindow(SW_HIDE);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_BUTTON_DREHYR)->ShowWindow(SW_HIDE);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_BUTTON_DREHZL)->ShowWindow(SW_HIDE);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_BUTTON_DREHZR)->ShowWindow(SW_HIDE);
	
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_SPINTAU)->ShowWindow(SW_HIDE);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_NUMPUNKTE_FELD)->ShowWindow(SW_HIDE);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_RESETVIEW)->ShowWindow(SW_HIDE);
		m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(ID_SAVE)->ShowWindow(SW_HIDE);
	}
}

//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Called during initial app startup, this function performs all the
//       permanent initialization.
//-----------------------------------------------------------------------------
HRESULT CRekoView::OneTimeSceneInit()
{
    // Set up our view matrix. A view matrix can be defined given an eye point,
    // a point to lookat, and a direction for which way is up. Here we look at the
    // origin, and define "up" to be in the z-direction.
    D3DXVECTOR3 vEye = D3DXVECTOR3( 2.0f, 3.0f, 0.5f );
    D3DXVECTOR3 vAt  = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
    D3DXVECTOR3 vUp  = D3DXVECTOR3( 0.0f, 0.0f, 1.0f );
    D3DXMatrixLookAtRH( &m_matView, &vEye, &vAt, &vUp );

    // Set the position matrix
    D3DXMatrixInverse( &m_matPosition, NULL, &m_matView );

		x							=	0.0f;
		y							= 0.0f;
		z							=	0.0f;

		return S_OK;
}

//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
HRESULT CRekoView::FrameMove()
{
    FLOAT fSecsPerFrame = 0.1f;//m_fElapsedTime;

    // Process keyboard input
    D3DXVECTOR3 vT( 0.0f, 0.0f, 0.0f );
    D3DXVECTOR3 vR( 0.0f, 0.0f, 0.0f );

    m_vVelocity        = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
    m_vAngularVelocity = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );

    if( m_bKey[VK_LEFT] || m_bKey[VK_NUMPAD1] )                 vT.x -= 1.0f; // Slide Left
    if( m_bKey[VK_RIGHT] || m_bKey[VK_NUMPAD3] )                vT.x += 1.0f; // Slide Right
    if( m_bKey[VK_DOWN] )                                       vT.y -= 1.0f; // Slide Down
    if( m_bKey[VK_UP] )                                         vT.y += 1.0f; // Slide Up

    if( m_bKey[VK_ADD] || m_bKey[187] )                         vT.z -= 2.0f; // Move Forward
    if( m_bKey[VK_SUBTRACT] || m_bKey[189] )                    vT.z += 2.0f; // Move Backward

    if( m_bKey['A'] || m_bKey[VK_NUMPAD8] )                     vR.x -= 1.0f; // Pitch Down
    if( m_bKey['Z'] || m_bKey[VK_NUMPAD2] )                     vR.x += 1.0f; // Pitch Up
    if( m_bKey['E'] || m_bKey[VK_NUMPAD4] )                     vR.y -= 1.0f; // Turn Right
    if( m_bKey['Q'] || m_bKey[VK_NUMPAD6] )                     vR.y += 1.0f; // Turn Left
    if( m_bKey[VK_NUMPAD9] )                                    vR.z -= 2.0f; // Roll CW
    if( m_bKey[VK_NUMPAD7] )                                    vR.z += 2.0f; // Roll CCW
    if( m_bKey['O'] )                                           x -= 3.0f; // Dreh um X+
    if( m_bKey['P'] )                                           x += 3.0f; // Dreh um X-
    if( m_bKey['K'] )                                           y -= 3.0f; // Dreh um Y+
		if( m_bKey['L'] )                                           y += 3.0f; // Dreh um Y-
    if( m_bKey['N'] )                                           z -= 3.0f; // Dreh um Z+
    if( m_bKey['M'] )                                           z += 3.0f; // Dreh um Z-

    m_vVelocity        = m_vVelocity * 0.9f + vT * 0.1f;
    m_vAngularVelocity = m_vAngularVelocity * 0.9f + vR * 0.1f;


    // Update position and view matricies
    D3DXMATRIXA16     matT, matR, matRotateX,matRotateY,matRotateZ;
    D3DXQUATERNION qR;

    vT = m_vVelocity * fSecsPerFrame * m_fSpeed;
    vR = m_vAngularVelocity * fSecsPerFrame * m_fAngularSpeed;

    D3DXMatrixTranslation( &matT, vT.x, vT.y, vT.z);
    D3DXMatrixMultiply( &m_matPosition, &matT, &m_matPosition );

    D3DXQuaternionRotationYawPitchRoll( &qR, vR.y, vR.x, vR.z );
    D3DXMatrixRotationQuaternion( &matR, &qR );
    D3DXMatrixMultiply( &m_matPosition, &matR, &m_matPosition );


    D3DXMatrixInverse( &m_matView, NULL, &m_matPosition );
		
		D3DXMatrixRotationX( &matRotateX, x*D3DX_PI/180 );
    D3DXMatrixMultiply( &m_matView, &matRotateX, &m_matView );

		D3DXMatrixRotationY( &matRotateY, y*D3DX_PI/180 );
    D3DXMatrixMultiply( &m_matView, &matRotateY, &m_matView );

		D3DXMatrixRotationZ( &matRotateZ, z*D3DX_PI/180 );
    D3DXMatrixMultiply( &m_matView, &matRotateZ, &m_matView );
    m_pd3dDevice->SetTransform( D3DTS_VIEW, &m_matView );

    return S_OK;
}


//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
HRESULT CRekoView::Render()
{
    // Clear the backbuffer with a black background
    m_pd3dDevice->Clear( 0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, m_clrBackBuffer, 1.0f, 0L );

    // Begin the scene
    if( SUCCEEDED( m_pd3dDevice->BeginScene() ) )
    {
    // Draw the points in the vertex buffer. This is broken into a few
    // steps. We are passing the vertices down a "stream", so first we need
    // to specify the source of that stream, which is our vertex buffer. Then
    // we need to let D3D know what vertex shader to use. Full, custom vertex
    // shaders are an advanced topic, but in most cases the vertex shader is
    // just the FVF, so that D3D knows what type of vertices we are dealing
    // with. Finally, we call DrawPrimitive() which does the actual rendering
    // of our geometry.

			// Wenn Neues Datenarray vorhanden, dann Buffer erstellen
			if (m_Recalc)
 				NeueRekoPunkte();

			// Koordinatensystem ausgeben
		  m_pd3dDevice->SetStreamSource( 0, m_pVBKoord, sizeof(REKOVERTEX) );
			m_pd3dDevice->SetVertexShader( D3DFVF_REKOVERTEX );
	    m_pd3dDevice->DrawPrimitive( D3DPT_LINELIST, 0, 3 );

      // Output statistics
			DWORD Textfarbe=D3DCOLOR_XRGB(GetRValue(m_clrText),GetGValue(m_clrText),GetBValue(m_clrText));

    if( m_bShowHelp )
    {
		  m_pFont->DrawText( 2,  0, Textfarbe, m_strFrameStats );//D3DCOLOR_ARGB(255,255,255,0)
		  m_pFont->DrawText( 2, 20, Textfarbe, m_strDeviceStats );
			CString Text;
			Text.LoadString(IDS_3DHELP_SHORTCUTS);
      m_pFontSmall->DrawText(  2, 40, Textfarbe, Text.GetBuffer(Text.GetLength()));
     	
			Text.LoadString(IDS_3DHELP_SHORTCUTS1);
 			m_pFontSmall->DrawText( 20, 60, Textfarbe, Text.GetBuffer(Text.GetLength()));

			Text.LoadString(IDS_3DHELP_SHORTCUTS2);
			m_pFontSmall->DrawText( 210, 60, Textfarbe, Text.GetBuffer(Text.GetLength()));
    }


			// Wenn Zeitreihen oder Wavearray gefllt, dann ausgeben
			if (m_dwNumVertices>1)
			{
		    m_pd3dDevice->SetStreamSource( 0, m_pVBZeitreihe, sizeof(REKOVERTEX) );
		    m_pd3dDevice->SetVertexShader( D3DFVF_REKOVERTEX );
				if (m_bIsPunkte) 
					m_pd3dDevice->DrawPrimitive( D3DPT_POINTLIST, 0, m_dwNumVertices );
				else
					m_pd3dDevice->DrawPrimitive( D3DPT_LINESTRIP, 0, m_dwNumVertices-1 );
			}

			// End the scene.
      m_pd3dDevice->EndScene();
    }
		CSoundsDoc* pDoc=GetDocument();
		m_pParentRekoFrame->m_wndDlgBar.SetDlgItemInt(IDC_NULLSTELLE,*pDoc->GetAutoKorrNullstelle());
    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
HRESULT CRekoView::InitDeviceObjects()
{
    m_pFont->InitDeviceObjects( m_pd3dDevice );
    m_pFontSmall->InitDeviceObjects( m_pd3dDevice );
    return S_OK;
}


//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Initialize scene objects.
//-----------------------------------------------------------------------------
HRESULT CRekoView::RestoreDeviceObjects()
{
    m_pFont->RestoreDeviceObjects();
    m_pFontSmall->RestoreDeviceObjects();

    // Setup render states
    m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );
		m_pd3dDevice->SetRenderState( D3DRS_POINTSIZE, *((DWORD*)&m_SizePts));

	// Desc: Creates a vertex buffer and fills it with our vertices. The vertex
	//       buffer is basically just a chuck of memory that holds vertices. After
	//       creating it, we must Lock()/Unlock() it to fill it. For indices, D3D
	//       also uses index buffers. The special thing about vertex and index
	//       buffers is that they can be created in device memory, allowing some
	//       cards to process them in hardware, resulting in a dramatic
	//       performance gain.


    // Koordinatensystem einlesen
		DWORD Textfarbe=D3DCOLOR_XRGB(GetRValue(m_clrText),GetGValue(m_clrText),GetBValue(m_clrText));

    REKOVERTEX m_Koordinatensystem[] =
    {
        { -2.0f,  0.0f, 0.0f,  Textfarbe, }, // x, y, z, color
        { +2.0f,  0.0f, 0.0f, Textfarbe, },
        {  0.0f, +2.0f, 0.0f, Textfarbe, },
        {  0.0f, -2.0f, 0.0f, Textfarbe, }, 
        {  0.0f,  0.0f, +2.0f,Textfarbe, },
        {  0.0f,  0.0f, -2.0f, Textfarbe, },
/*
        { -2.0f,  0.0f, 0.0f, m_SizePts, Textfarbe, }, // x, y, z, color
        { +2.0f,  0.0f, 0.0f, m_SizePts, Textfarbe, },
        {  0.0f, +2.0f, 0.0f, m_SizePts, Textfarbe, },
        {  0.0f, -2.0f, 0.0f, m_SizePts, Textfarbe, }, 
        {  0.0f,  0.0f, +2.0f, m_SizePts, Textfarbe, },
        {  0.0f,  0.0f, -2.0f, m_SizePts, Textfarbe, },
*/
    };
		// Create the vertex buffer. Here we are allocating enough memory
    // (from the default pool) to hold all our 6 custom vertices. We also
    // specify the FVF, so the vertex buffer knows what data it contains.
//		if (m_pVBKoord!=NULL) 
//				m_pVBKoord->Release();
		SAFE_RELEASE(m_pVBKoord);
		if( FAILED( m_pd3dDevice->CreateVertexBuffer( 6*sizeof(REKOVERTEX), D3DUSAGE_WRITEONLY, D3DFVF_REKOVERTEX,
                                                  D3DPOOL_DEFAULT, &m_pVBKoord ) ) )
        return E_FAIL;
    // Now we fill the vertex buffer. To do this, we need to Lock() the VB to
    // gain access to the vertices. This mechanism is required because vertex
    // buffers may be in device memory.
    VOID* pVerticesKoords;
    if( FAILED( m_pVBKoord->Lock( 0, sizeof(m_Koordinatensystem), (BYTE**)&pVerticesKoords, 0 ) ) )
        return E_FAIL;
    memcpy( pVerticesKoords, m_Koordinatensystem, sizeof(m_Koordinatensystem) );
    m_pVBKoord->Unlock();


		// Dasselbe nochmal zur Initialisierung auch fr den Puffer mit den Zeitreihenwerten
		// Zur Initialisierung aber erst einen Dummybuffer herstellen.
//		if (m_pVBZeitreihe!=NULL) 
//				m_pVBZeitreihe->Release();
		SAFE_RELEASE(m_pVBZeitreihe);
		if( FAILED( m_pd3dDevice->CreateVertexBuffer( 1*sizeof(REKOVERTEX), D3DUSAGE_WRITEONLY, D3DFVF_REKOVERTEX,
                                                  D3DPOOL_DEFAULT, &m_pVBZeitreihe ) ) )
        return E_FAIL;

		NeueRekoPunkte();

    // Set up the projection matrix
    FLOAT fAspectRatio = (FLOAT)m_d3dsdBackBuffer.Width / (FLOAT)m_d3dsdBackBuffer.Height;
    D3DXMatrixPerspectiveFovRH( &m_matProj, D3DXToRadian(60.0f), fAspectRatio, 0.1f, 100.0f );
    m_pd3dDevice->SetTransform( D3DTS_PROJECTION, &m_matProj );

    return S_OK;
}

//-----------------------------------------------------------------------------
// Name: NeueRekoPunkte()
// Desc: Wenn die Arrays neu aufgefllt wurden, neue Punkte einlesen.
//-----------------------------------------------------------------------------
HRESULT CRekoView::NeueRekoPunkte()
	{
    // Koordinatensystem einlesen
		DWORD Textfarbe=D3DCOLOR_XRGB(GetRValue(m_clrText),GetGValue(m_clrText),GetBValue(m_clrText));

    REKOVERTEX m_Koordinatensystem[] =
    {
        { -2.0f,  0.0f, 0.0f,  Textfarbe, }, // x, y, z, color
        { +2.0f,  0.0f, 0.0f, Textfarbe, },
        {  0.0f, +2.0f, 0.0f, Textfarbe, },
        {  0.0f, -2.0f, 0.0f, Textfarbe, }, 
        {  0.0f,  0.0f, +2.0f,Textfarbe, },
        {  0.0f,  0.0f, -2.0f, Textfarbe, },
    };
		SAFE_RELEASE(m_pVBKoord);
		if( FAILED( m_pd3dDevice->CreateVertexBuffer( 6*sizeof(REKOVERTEX), D3DUSAGE_WRITEONLY, D3DFVF_REKOVERTEX,
                                                  D3DPOOL_DEFAULT, &m_pVBKoord ) ) )
        return E_FAIL;
    VOID* pVerticesKoords;
    if( FAILED( m_pVBKoord->Lock( 0, sizeof(m_Koordinatensystem), (BYTE**)&pVerticesKoords, 0 ) ) )
        return E_FAIL;
    memcpy( pVerticesKoords, m_Koordinatensystem, sizeof(m_Koordinatensystem) );
    m_pVBKoord->Unlock();

		// Zeitreihenarray als Punkte einlesen
		CSoundsDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);
		INT* pBitRate=pDoc->GetBitRate();
		BOOL* pSigned=pDoc->GetSigned();
		CArray<DOUBLE,DOUBLE>* pZeitreiheArray=pDoc->GetZeitreiheArray();
		DOUBLE MaxZeit=pZeitreiheArray->GetSize()-1;
		DOUBLE* pMaxAmplitude=pDoc->GetMaxAmplitude();
		if (MaxZeit>(2*m_Tau)) // Wenn Array berhaupt belegt.
		{
			//Rescale WAV Array auf -1 bis 1
			FLOAT Messpunkt;
			FLOAT FaktorX=0,FaktorY=0;

 			m_RekoPtArray.RemoveAll();

			INT Sprung=1;

			//Anzahl der dargestellten Punkte auf m_NumRekoPoints reduzieren auch bei groen FFTs
			if (MaxZeit>m_NumRekoPoints)
				Sprung=(INT)(ceil(MaxZeit/m_NumRekoPoints));
			else
				Sprung=1;

			m_RekoPtArray.SetSize((INT)(MaxZeit/Sprung));


			m_dwNumVertices=(DWORD)(fabs(MaxZeit/Sprung-2*m_Tau));

			FaktorY=(FLOAT)(2/(*pMaxAmplitude));
			INT i=0;
			if (*pDoc->GetSigned())
			{
				while (i<(INT)(MaxZeit/Sprung))
					{
				    Messpunkt=(FLOAT)(pZeitreiheArray->GetAt(i*Sprung)*FaktorY);
//					  m_RekoPtArray.Add(Messpunkt);
					  m_RekoPtArray[i]=Messpunkt;
						i++;
		      }
			}
			else
			{	
				while (i<(INT)(MaxZeit/Sprung))
					{
				    Messpunkt=(FLOAT)(pZeitreiheArray->GetAt(i*Sprung)*FaktorY-1);
//					  m_RekoPtArray.Add(Messpunkt);
					  m_RekoPtArray[i]=Messpunkt;
						i++;
		      }
			}

			// Alten Puffer oder Dummybuffer der Initialisierung entfernen
//		if (m_pVBZeitreihe!=NULL) 
//				m_pVBZeitreihe->Release();
			SAFE_RELEASE(m_pVBZeitreihe);

			// Punktebuffer erstellen
			if( FAILED( m_pd3dDevice->CreateVertexBuffer( m_dwNumVertices*sizeof(REKOVERTEX),
                                                  D3DUSAGE_WRITEONLY, D3DFVF_REKOVERTEX,
                                                  D3DPOOL_DEFAULT, &m_pVBZeitreihe ) ) )
			    return E_FAIL;

			// Punkte in Puffer einlesen
			REKOVERTEX *pVerticesZeitreihe;
			if( FAILED( m_pVBZeitreihe->Lock( 0, m_dwNumVertices*sizeof(REKOVERTEX), (BYTE**)&pVerticesZeitreihe, 0 ) ) )
				  return E_FAIL;

//			CArray<REKOVERTEX,REKOVERTEX> m_VerticesZeitreihe;
			REKOVERTEX Puffer;
			for( DWORD y=0; y<m_dwNumVertices; y++ )
			{
				Puffer.x=m_RekoPtArray.GetAt(y);
				Puffer.y=m_RekoPtArray.GetAt(y+m_Tau);
				Puffer.z=m_RekoPtArray.GetAt(y+2*m_Tau);
//				Puffer.ptsize=m_SizePts;
				if (m_bIsBunt)
					Puffer.color=Farbwahl((INT)(y*1663/m_dwNumVertices));
				else
					Puffer.color=m_clrReko;
				pVerticesZeitreihe[y]=Puffer;
				//				m_VerticesZeitreihe.Add(Puffer);
			}

/*
			for( DWORD x=0; x<m_dwNumVertices; x++ )
      {
				pVerticesZeitreihe[x] = m_VerticesZeitreihe.GetAt(x);
      }
*/

			m_pVBZeitreihe->Unlock();
		}// wenn berhaupt belegt
		else
		{
			m_RekoPtArray.RemoveAll();
			m_RekoPtArray.Add(0);
			// Alten Puffer oder Dummybuffer der Initialisierung entfernen
//		if (m_pVBZeitreihe!=NULL) 
//				m_pVBZeitreihe->Release();
			SAFE_RELEASE(m_pVBZeitreihe);
			m_dwNumVertices=1;
			// Dummybuffer erstellen
			if( FAILED( m_pd3dDevice->CreateVertexBuffer( m_dwNumVertices*sizeof(REKOVERTEX),
                                                  D3DUSAGE_WRITEONLY, D3DFVF_REKOVERTEX,
                                                  D3DPOOL_DEFAULT, &m_pVBZeitreihe ) ) )
			    return E_FAIL;
			m_dwNumVertices=0;
		}

		return S_OK;
}

//-----------------------------------------------------------------------------
// Name: Farbwahl ()
// Desc: Farbspiel einbauen.
//-----------------------------------------------------------------------------
DWORD CRekoView::Farbwahl(INT counter)
{
	INT r=0,g=0,b=0;
	if (counter<128) {
		r=128+counter;
		g=0;
		b=0;
	}
	if ((counter>=128)&&(counter<384)) {
		r=255;
		g=counter-128;
		b=0;
	}
	if ((counter>=384)&&(counter<640)) {
		r=639-counter;
		g=255;
		b=0;
	}
	if ((counter>=640)&&(counter<896)) {
		r=0;
		g=255;
		b=counter-640;
	}
	if ((counter>=896)&&(counter<1152)) {
		r=0;
		g=1151-counter;
		b=255;
	}
	if ((counter>=1152)&&(counter<1408)) {
		r=counter-1152;
		g=0;
		b=255;
	}
	if ((counter>=1408)&&(counter<1664)) {
		r=255;
		g=counter-1408;
		b=255;
	}
	return D3DCOLOR_XRGB(r,g,b);  // hier evtl. Farbspiel einbauen

}


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Called when the device-dependent objects are about to be lost.
//-----------------------------------------------------------------------------
HRESULT CRekoView::InvalidateDeviceObjects()
{
    m_pFont->InvalidateDeviceObjects();
    m_pFontSmall->InvalidateDeviceObjects();
//		if (m_pVBZeitreihe!=NULL) 
//				m_pVBZeitreihe->Release();
//		if (m_pVBKoord!=NULL) 
//				m_pVBKoord->Release();
    SAFE_RELEASE( m_pVBKoord );
    SAFE_RELEASE( m_pVBZeitreihe);

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
HRESULT CRekoView::DeleteDeviceObjects()
{
    m_pFont->DeleteDeviceObjects();
    m_pFontSmall->DeleteDeviceObjects();
    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
HRESULT CRekoView::FinalCleanup()
{
		m_RekoPtArray.RemoveAll();
    SAFE_DELETE( m_pFont );
    SAFE_DELETE( m_pFontSmall );
    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device intialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
HRESULT CRekoView::ConfirmDevice( D3DCAPS8* pCaps, DWORD dwBehavior, 
                                 D3DFORMAT Format )
{
    if( (dwBehavior & D3DCREATE_HARDWARE_VERTEXPROCESSING ) ||
        (dwBehavior & D3DCREATE_MIXED_VERTEXPROCESSING ) )
    {
        if( pCaps->VertexShaderVersion < D3DVS_VERSION(1,0) )
            return E_FAIL;
    }

    return S_OK;
}



void CRekoView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	if (lHint==50||g_bIsCapturing||g_bIsPlayingWave||g_bIsPutPoints)
	{
		m_Recalc=TRUE;
		Render3DEnvironment();	
	}
}

void CRekoView::OnSize(UINT nType, int cx, int cy) 
{
	if (m_bReady==1) 
	{

		// Halt frame movement while the app is sizing or moving
		Pause( TRUE );

		// Check to see if we are losing our window...
		if( SIZE_MAXHIDE==nType || SIZE_MINIMIZED==nType )
	  {	
		  if( m_bClipCursorWhenFullscreen && !m_bWindowed )
			   ClipCursor( NULL );
	    m_bActive = FALSE;
		}
	  else
		  m_bActive = TRUE;

	  Pause( FALSE );

		if( m_bActive && m_bWindowed )
	  {
		  RECT rcClientOld;
			rcClientOld = m_rcWindowClient;

			// Update window properties
		  GetWindowRect( &m_rcWindowBounds );
	    GetClientRect( &m_rcWindowClient );

	    if( rcClientOld.right - rcClientOld.left != m_rcWindowClient.right - m_rcWindowClient.left ||
			    rcClientOld.bottom - rcClientOld.top != m_rcWindowClient.bottom - m_rcWindowClient.top)
		  {
				// A new window size will require a new backbuffer
				// size, so the 3D structures must be changed accordingly.
			  m_bReady = FALSE;

		    m_d3dpp.BackBufferWidth  = m_rcWindowClient.right - m_rcWindowClient.left;
	      m_d3dpp.BackBufferHeight = m_rcWindowClient.bottom - m_rcWindowClient.top;

		    // Resize the 3D environment
	      if( FAILED( Resize3DEnvironment() ) )
        DisplayErrorMsg( D3DAPPERR_RESIZEFAILED, MSGERR_APPMUSTEXIT );

				m_bReady = TRUE;
			}
		}
	}
//	m_NumRekoPoints=(INT)(cx*cy/100);
} 

void CRekoView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	m_bKey[nChar]=1;
	Render3DEnvironment();

	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CRekoView::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	m_bKey[nChar]=0;

	CView::OnKeyUp(nChar, nRepCnt, nFlags);
}


void CRekoView::OnTogglehelp() 
{
  m_bShowHelp = !m_bShowHelp;
	Render3DEnvironment();
}

void CRekoView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView) 
{
  m_bHasFocus = bActivate;
	
	CView::OnActivateView(bActivate, pActivateView, pDeactiveView);
}

void CRekoView::OnTogglestart() 
{
  // Toggle frame movement
  m_bFrameMoving = !m_bFrameMoving;
  DXUtil_Timer( m_bFrameMoving ? TIMER_START : TIMER_STOP );
}


void CRekoView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	m_ptPrev = point;
	CView::OnLButtonDown(nFlags, point);
}

void CRekoView::OnMouseMove(UINT nFlags, CPoint point) 
{
	m_Recalc=FALSE;
	// Wird aufgerufen, wenn die Maus bewegt wird.
  if (nFlags==MK_RBUTTON)
  {
	  if ((m_ptPrev.x-point.x)<0) DoFrameMove(VK_NUMPAD6); //Maus nach links
		if ((m_ptPrev.x-point.x)>0) DoFrameMove(VK_NUMPAD4); //Maus nach rechts
	  if ((m_ptPrev.y-point.y)<0) DoFrameMove(VK_NUMPAD2); //Maus nach unten
		if ((m_ptPrev.y-point.y)>0) DoFrameMove(VK_NUMPAD8); //Maus nach oben
	}

  if (nFlags==MK_LBUTTON)
  {
	  if ((m_ptPrev.x-point.x)<0) DoFrameMove(VK_LEFT);//Maus nach links
	  if ((m_ptPrev.x-point.x)>0) DoFrameMove(VK_RIGHT);//Maus nach rechts
	  if ((m_ptPrev.y-point.y)<0) DoFrameMove(VK_UP); //Maus nach unten
	  if ((m_ptPrev.y-point.y)>0) DoFrameMove(VK_DOWN);//Maus nach oben
  }

/*	for(int i=0; i<256; i++)
	{
		m_bKey[i]=0;
	}
*/
	m_ptPrev = point;
	CView::OnMouseMove(nFlags, point);
}

void CRekoView::OnSinglestep() 
{
  // Single-step frame movement
  if( FALSE == m_bFrameMoving )
    DXUtil_Timer( TIMER_ADVANCE );
  else
    DXUtil_Timer( TIMER_STOP );
  m_bFrameMoving = FALSE;
  m_bSingleStep  = TRUE;
}

void CRekoView::OnChangedevice() 
{
  HRESULT hr;
  // Prompt the user to select a new device or mode
  if( m_bActive && m_bReady )
  {
		m_bReady=FALSE;
		//    Pause(TRUE);

    // Can't display dialogs in fullscreen mode
    if( m_bWindowed == FALSE )
    {
        if( FAILED( ToggleFullscreen() ) )
        {
            DisplayErrorMsg( D3DAPPERR_RESIZEFAILED, MSGERR_APPMUSTEXIT );
        }
    }

		CGrafikDeviceDlg Dlg;
		Dlg.m_RekoView=this;
		Dlg.m_bReko=TRUE;
		if (Dlg.DoModal()==IDOK) 
		{

	    // Get access to the newly selected adapter, device, and mode
			DWORD dwDevice;
		  dwDevice  = m_Adapters[m_dwAdapter].dwCurrentDevice;
	    m_bWindowed = m_Adapters[m_dwAdapter].devices[dwDevice].bWindowed;

			// Release all scene objects that will be re-created for the new device
		  InvalidateDeviceObjects();
	    DeleteDeviceObjects();

		  // Release display objects, so a new device can be created
	    if( m_pd3dDevice->Release() > 0L )
        DisplayErrorMsg( D3DAPPERR_NONZEROREFCOUNT, MSGERR_APPMUSTEXIT );

		  // Inform the display class of the change. It will internally
	    // re-create valid surfaces, a d3ddevice, etc.
			CSoundsDoc* pDoc=GetDocument();
	    if( FAILED( hr = Initialize3DEnvironment() ) )
			{
        DisplayErrorMsg( hr, MSGERR_APPMUSTEXIT );
				m_bReady=FALSE;
				pDoc->m_bIsNot3D=TRUE;
				goto Stop;
			}


			// If the app is paused, trigger the rendering of the current frame
		  if( FALSE == m_bFrameMoving )
			{
        m_bSingleStep = TRUE;
        DXUtil_Timer( TIMER_START );
        DXUtil_Timer( TIMER_STOP );
	    }
    }
		m_bReady=TRUE;
//    Pause(FALSE);
		Render3DEnvironment();
  }
Stop:
	;
}

BOOL CRekoView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	
	if (nFlags==MK_CONTROL) 
	{
		if (zDelta>0) 
		  DoFrameMove('K');
		else
		  DoFrameMove('L');
	}
	if (nFlags==MK_SHIFT) 
	{
		if (zDelta>0) 
		  DoFrameMove('O');
		else
		  DoFrameMove('P');
	}
	if (nFlags==0) 
	{
		if (zDelta>0) 
		  DoFrameMove('N');
		else
		  DoFrameMove('M');
	}
	return CView::OnMouseWheel(nFlags, zDelta, pt);
}

void CRekoView::OnChangePunkteFeld() 
{
	INT SizePts=m_pParentRekoFrame->m_wndDlgBar.GetDlgItemInt(IDC_PUNKTE_FELD);
	m_SizePts=(float)(SizePts*0.5);
//	m_Recalc=TRUE;
	if (m_pd3dDevice!=NULL) 
		m_pd3dDevice->SetRenderState(D3DRS_POINTSIZE, *((DWORD*)&m_SizePts));
	if (m_bReady) 
	 	Render3DEnvironment();
}

void CRekoView::OnFarbwahl() 
{
	CColorDialog dlgColor(m_clrReko);
	if (dlgColor.DoModal() == IDOK)
	{
		m_clrRekoREF = dlgColor.GetColor();
		m_pRadioFesteFarbe=(CButton*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_FESTEFARBE);
		m_pRadioFesteFarbe->SetCheck(1);
		m_pRadioBunt=(CButton*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_BUNT);
		m_pRadioBunt->SetCheck(0);
		m_clrReko=D3DCOLOR_XRGB(GetRValue(m_clrRekoREF),GetGValue(m_clrRekoREF),GetBValue(m_clrRekoREF));
		m_FarbButton.SetColour(m_clrText,m_clrRekoREF);
		m_bIsBunt=FALSE;
		if (m_bReady) 
		 	Render3DEnvironment();
	}

}

void CRekoView::OnBkgclr() 
{
	CColorDialog dlgColor(m_clrBkg);
	if (dlgColor.DoModal() == IDOK)
	{
		m_clrBkg = dlgColor.GetColor();
		m_clrBackBuffer=D3DCOLOR_XRGB(GetRValue(m_clrBkg),GetGValue(m_clrBkg),GetBValue(m_clrBkg));
		m_clrText=TRGB(GetRValue(m_clrBkg),GetGValue(m_clrBkg),GetBValue(m_clrBkg));
		m_FarbButtonBkg.SetColour(m_clrText,m_clrBkg);
		m_Recalc=TRUE;
		if (m_bReady) 
		 	Render3DEnvironment();
	}
}

void CRekoView::OnRenderbtn() 
{
	m_Recalc=TRUE;
	if (m_bReady) 
	 	Render3DEnvironment();
}

void CRekoView::OnBunt() 
{
	m_FarbButton.SetColour(GetSysColor(COLOR_INACTIVECAPTION),GetSysColor(COLOR_MENU));
	m_bIsBunt=TRUE;
	m_Recalc=TRUE;
	if (m_bReady) 
 		Render3DEnvironment();
}

void CRekoView::OnFestefarbe() 
{
	m_FarbButton.SetColour(m_clrText,m_clrRekoREF);
	m_clrReko=D3DCOLOR_XRGB(GetRValue(m_clrRekoREF),GetGValue(m_clrRekoREF),GetBValue(m_clrRekoREF));
	m_bIsBunt=FALSE;
	m_Recalc=TRUE;
 	if (m_bReady) 
		Render3DEnvironment();
}

void CRekoView::OnChangeTau() 
{
	m_Tau=m_pParentRekoFrame->m_wndDlgBar.GetDlgItemInt(IDC_TAU);
	m_Recalc=TRUE;
	if (m_bReady) 
	 	Render3DEnvironment();
}

void CRekoView::DoFrameMove(char Buchstabe) 
{
	m_bKey[Buchstabe]=1;	
	if (m_bReady) 
	{
		FrameMove();
		Render3DEnvironment();
	}
	m_bKey[Buchstabe]=0;	
}


void CRekoView::OnButtonDrehxl() 
{
	DoFrameMove('O');
}

void CRekoView::OnButtonDrehxr() 
{
	DoFrameMove('P');
}

void CRekoView::OnButtonDrehyl() 
{
	DoFrameMove('K');
}

void CRekoView::OnButtonDrehyr() 
{
	DoFrameMove('L');
}

void CRekoView::OnButtonDrehzl() 
{
	DoFrameMove('M');
}

void CRekoView::OnButtonDrehzr() 
{
	DoFrameMove('N');
}

void CRekoView::OnResetview() 
{
	if (m_bReady) 
	{
		OneTimeSceneInit();
		Render3DEnvironment();
	}
}

void CRekoView::OnPunkte() 
{
	m_pPunkteWertCtrl=(CEdit*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_PUNKTE_FELD);
	m_pPunkteWertCtrl->EnableWindow(TRUE);
	m_pRadioLinien=(CButton*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_LINIEN);
	m_pRadioLinien->SetCheck(0);
	m_pRadioPunkte=(CButton*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_PUNKTE);
	m_pRadioPunkte->SetCheck(1);
	m_bIsPunkte=TRUE;
	m_SizePts=1.0f;
	if (m_bReady) 
 		Render3DEnvironment();
}

void CRekoView::OnLinien() 
{
	m_pPunkteWertCtrl=(CEdit*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_PUNKTE_FELD);
	m_pPunkteWertCtrl->EnableWindow(FALSE);
	m_pRadioLinien=(CButton*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_LINIEN);
	m_pRadioLinien->SetCheck(1);
	m_pRadioPunkte=(CButton*)m_pParentRekoFrame->m_wndDlgBar.GetDlgItem(IDC_PUNKTE);
	m_pRadioPunkte->SetCheck(0);
	m_bIsPunkte=FALSE;
	m_Recalc=TRUE;
	m_SizePts=1.0f;
	if (m_bReady) 
 		Render3DEnvironment();
}

void CRekoView::OnSave() 
{
	HRESULT hr;
	CFileDialog FotoFileDlg(FALSE,NULL,NULL,OFN_OVERWRITEPROMPT|OFN_NOREADONLYRETURN|OFN_ENABLESIZING,\
              "BMP - Bitmap|*.bmp|DDS - DirectDraw Surface File|*.dds|");
  FotoFileDlg.m_ofn.nFilterIndex=1;
	FotoFileDlg.m_ofn.lpstrDefExt=".bmp";
	CString Text;
	Text.LoadString(IDS_SAVE_REKO);
	FotoFileDlg.m_ofn.lpstrTitle=Text.GetBuffer(Text.GetLength());
  if (FotoFileDlg.DoModal()==IDOK) 
	{
		CString Pfad=FotoFileDlg.GetPathName();
		LPTSTR strFileName=Pfad.GetBuffer(MAX_PATH);
		// get the backbuffer description
	  LPDIRECT3DSURFACE8 backbuf;
		
		if(FAILED(hr=m_pd3dDevice->GetBackBuffer(0,D3DBACKBUFFER_TYPE_MONO,&backbuf))) 
       goto Ende;

	  D3DSURFACE_DESC desc;
		hr=backbuf->GetDesc(&desc);
		if(FAILED(hr))
       goto Ende;
	  // create the image buffer
		LPDIRECT3DSURFACE8 surf;
	  if (FAILED(hr=m_pd3dDevice->CreateImageSurface(desc.Width,desc.Height,desc.Format,&surf))) 
       goto Ende;
		// copy the image buffer and release the back buffer
	  hr=D3DXLoadSurfaceFromSurface(surf,NULL,NULL,backbuf,NULL,NULL,D3DX_FILTER_NONE,0);
		backbuf->Release();
	  if (FAILED(hr)) 
			surf->Release();
		else
		{
			D3DXIMAGE_FILEFORMAT FileFormat;
			if (FotoFileDlg.m_ofn.nFilterIndex==1) 
				FileFormat=D3DXIFF_BMP;
			else
				FileFormat=D3DXIFF_DDS;
			
			//Finally, we call D3DXSaveSurfaceToFile() to create the file, 
			//and release the temporary image surface:
			hr=D3DXSaveSurfaceToFile(strFileName,FileFormat,surf,NULL,NULL);

			// release the image surface
			surf->Release();
		}
	}
Ende:
	;
}


void CRekoView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
	// TODO: Speziellen Code hier einfgen und/oder Basisklasse aufrufen
	
	CView::OnPrint(pDC, pInfo);
}

void CRekoView::OnEditClearall() 
{
	CSoundsDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	pDoc->DeleteContents();
	OnResetview();
	Invalidate(TRUE);
	pDoc->UpdateAllViews(NULL,50);
}

void CRekoView::OnEditCopy() 
{
	CDC* pDC=GetDC();
	CMetaFileDC * pMetaDC = new CMetaFileDC();
	pMetaDC->CreateEnhanced(GetDC(),NULL,NULL,"SOUNDS Soundanalyse\0Rekonstruktion Abbild\0\0");
	//draw meta file
	CRect rect;
	GetClientRect(rect);

	pMetaDC->BitBlt(0,0,rect.right,rect.bottom,pDC,0,0,SRCCOPY);
	//close meta file dc and prepare for clipboard;
	HENHMETAFILE hMF = pMetaDC->CloseEnhanced();

	//copy to clipboard
	// Open the clipboard...
	if(!OpenClipboard()) 
	{
    AfxMessageBox(IDS_OPEN_CLIPBOARD_FAIL);
    return;
	}
	EmptyClipboard();
	::SetClipboardData(CF_ENHMETAFILE,hMF);
	CloseClipboard();
	//DeleteMetaFile(hMF);
	delete pMetaDC;
}


BOOL CRekoView::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	CSoundsDoc* pDoc=GetDocument();
	pDoc->m_help.SetHelpFile(pDoc->m_strHelpFile);
	pDoc->m_help.DisplayTopic("html/Rekonstruktionsfenster.html");
	
	return CView::OnHelpInfo(pHelpInfo);
}


void CRekoView::CalcWindowRect(LPRECT lpClientRect, UINT nAdjustType) 
{
	if (lpClientRect->left==lpClientRect->right) 
	{
		lpClientRect->right+=1;
	}
	if (lpClientRect->top==lpClientRect->bottom) 
	{
		lpClientRect->bottom+=1;
	}
	
	CView::CalcWindowRect(lpClientRect, nAdjustType);
}

void CRekoView::OnChangeNumpunkteFeld() 
{
	m_NumRekoPoints=m_pParentRekoFrame->m_wndDlgBar.GetDlgItemInt(IDC_NUMPUNKTE_FELD);
	if (m_NumRekoPoints<=2*m_Tau) 
	{
		m_NumRekoPoints=3*m_Tau;
		m_pParentRekoFrame->m_wndDlgBar.SetDlgItemInt(IDC_NUMPUNKTE_FELD,m_NumRekoPoints);
	}
	m_Recalc=TRUE;
	if (m_bReady) 
	 	Render3DEnvironment();
}

int CRekoView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Speziellen Erstellungscode hier einfgen
	
	return 0;
}

void CRekoView::OnExportdata() 
{
	if (!g_bIsPlayingWave||g_bIsCapturing) 
	{
	BeginWaitCursor();
	//Daten berechnen aus Frequenzarray und als Tabulator getrennte Werte in CString schreiben
	CString Ausgabe;
	char buffer[30];
	CString strX;
	CString strY1,strY2,strY3;

	//1.Zeile schreiben
	Ausgabe.LoadString(IDS_EXPORT_REKO);

	CSoundsDoc* pDoc=GetDocument();
	CArray<DOUBLE,DOUBLE>* pZeitreiheArray=pDoc->GetZeitreiheArray();
	INT SamplingRate=*pDoc->GetSampleRate();
	INT MaxZeit=pZeitreiheArray->GetSize();
	INT MaxAmplitude=*pDoc->GetMaxAmplitude();

	for	(INT i=0; i<MaxZeit-2*m_Tau;i++)
	{
		DOUBLE xWert=i/(FLOAT)SamplingRate*1000;
		DOUBLE Pegel1=pZeitreiheArray->GetAt(i)/MaxAmplitude*100; //in Prozent
		DOUBLE Pegel2=pZeitreiheArray->GetAt(i+m_Tau)/MaxAmplitude*100; //in Prozent
		DOUBLE Pegel3=pZeitreiheArray->GetAt(i+2*m_Tau)/MaxAmplitude*100; //in Prozent
		strX=gcvt( xWert, 10, buffer);
//		strX=strX.Left(strX.Find(".")+5); 
    strX.Replace(".",",");
		strY1=gcvt( Pegel1, 10, buffer);
//		strY1=strY1.Left(strY1.Find(".")+5); 
    strY1.Replace(".",",");
		strY2=gcvt( Pegel2, 10, buffer);
//		strY2=strY2.Left(strY2.Find(".")+5); 
    strY2.Replace(".",",");
		strY3=gcvt( Pegel3, 10, buffer);
//		strY3=strY3.Left(strY3.Find(".")+5); 
    strY3.Replace(".",",");
		Ausgabe+=strX+_T("\t")+strY1+_T("\t")+strY2+_T("\t")+strY3+_T("\r\n");
	}
	
	// Open the clipboard...
	if(!OpenClipboard()) 
	{
    AfxMessageBox(IDS_OPEN_CLIPBOARD_FAIL);
    return;
	}

	// Empty anything already there...
	EmptyClipboard();


	// Allocate global memory for transfer...
	HGLOBAL hText = GlobalAlloc(GMEM_MOVEABLE |GMEM_DDESHARE, strlen(Ausgabe)+4);

	// Put our string in the global memory...
	char* ptr = (char *)GlobalLock(hText);
	strcpy(ptr, Ausgabe);
	GlobalUnlock(hText);

	// Put data on the clipboard as CF_TEXT
	::SetClipboardData(CF_TEXT, hText);

	// Free memory...
	GlobalFree(hText);

	// Close clipboard...
	CloseClipboard();

	//Besttigung ausgeben
	AfxMessageBox(IDS_REKOVIEW_ONEXPORTDATA,MB_ICONINFORMATION);
	EndWaitCursor();
	}
}

