// SonoView.cpp: Implementierungsdatei
//

#include "stdafx.h"
#include "Sounds.h"
#include "SonoView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define HEIGHT_FREQ_RULER 45
#define HEIGHT_KLAVIER 25

/////////////////////////////////////////////////////////////////////////////
// CSonoView

IMPLEMENT_DYNCREATE(CSonoView, CView)

CSonoView::CSonoView()
{
	INT Items=AfxGetMainWnd()->GetMenu()->GetMenuItemCount();
	AfxGetMainWnd()->GetMenu()->GetSubMenu(Items>5?5:3)->AppendMenu(MF_STRING|MF_ENABLED,ID_FENSTER_SONO,"Sonogramm");
	AfxGetMainWnd()->DrawMenuBar();
}

void CSonoView::IniFileRead()
{
	//INIFile Werte einlesen
	m_BottomCutHz=*theApp.GetIniSonoBottomCutHz();
	m_Liniendicke=*theApp.GetIniSonoLiniendicke();
 	m_bIsSmooth=*theApp.GetIniSonobIsSmooth();
	m_TopCutkHz=*theApp.GetIniSonoTopCutkHz();
	m_bIsAmpLinear=*theApp.GetIniSonobIsAmpLinear();
	m_bIsFreqLinear=*theApp.GetIniSonobIsFreqLinear();
	m_bIsFarbig1=*theApp.GetIniSonobIsFarbig1();
}

void	CSonoView::IniFileWrite()
{
	//INIFile Werte schreiben
	*theApp.GetIniSonoBottomCutHz()=m_BottomCutHz;
	*theApp.GetIniSonoLiniendicke()=m_Liniendicke;
 	*theApp.GetIniSonobIsSmooth()=m_bIsSmooth;
	*theApp.GetIniSonoTopCutkHz()=m_TopCutkHz;
	*theApp.GetIniSonobIsAmpLinear()=m_bIsAmpLinear;
	*theApp.GetIniSonobIsFreqLinear()=m_bIsFreqLinear;
	*theApp.GetIniSonobIsFarbig1()=m_bIsFarbig1;
}

CSonoView::~CSonoView()
{
	IniFileWrite();
	INT Items=AfxGetMainWnd()->GetMenu()->GetMenuItemCount();
	AfxGetMainWnd()->GetMenu()->GetSubMenu(Items>5?5:3)->RemoveMenu(ID_FENSTER_SONO,MF_BYCOMMAND);
	AfxGetMainWnd()->DrawMenuBar();
	CSoundsDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	pDoc->m_pViewSono=NULL;
}


BEGIN_MESSAGE_MAP(CSonoView, CView)
	//{{AFX_MSG_MAP(CSonoView)
	ON_WM_ERASEBKGND()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_SETFOCUS()
	ON_WM_SIZE()
	ON_EN_CHANGE(IDC_BOTTOMCUT_FELD, OnChangeBottomcutFeld)
	ON_BN_CLICKED(IDC_PALETTE_FARBIG, OnPaletteFarbig)
	ON_BN_CLICKED(IDC_SCALE_AMP_LIN, OnScaleAmpLin)
	ON_BN_CLICKED(IDC_SCALE_AMP_LOG, OnScaleAmpLog)
	ON_BN_CLICKED(IDC_SCALE_FREQ_LIN, OnScaleFreqLin)
	ON_BN_CLICKED(IDC_SCALE_FREQ_LOG, OnScaleFreqLog)
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_SLIDER_ZOOM_V, OnReleasedcaptureSliderZoomV)
	ON_EN_CHANGE(IDC_TOPCUT_FELD, OnChangeTopcutFeld)
	ON_EN_CHANGE(IDC_LINIENDICKE_FELD, OnChangeLiniendickeFeld)
	ON_COMMAND(ID_SAVE, OnSave)
	ON_COMMAND(ID_EDIT_CLEARALL, OnEditClearall)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_BN_CLICKED(IDC_PALETTE_FARBIG2, OnPaletteFarbig2)
	ON_BN_CLICKED(IDC_CHECK_SMOOTH, OnCheckSmooth)
	ON_WM_HELPINFO()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Diagnose CSonoView

#ifdef _DEBUG
void CSonoView::AssertValid() const
{
	CView::AssertValid();
}

void CSonoView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// Behandlungsroutinen fr Nachrichten CSonoView 

void CSonoView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();

	if (*theApp.GetIniAllgemeinGrundschulprofil())
	{
		CSoundsDoc* pDoc=GetDocument();
		*pDoc->GetFFTPoints()=8192;
		pDoc->UpdateStatusBar();
	}

	CRect rect;
	GetClientRect(rect);

	m_clrBkg=RGB(0,0,0);
	m_clrText=RGB(255,255,255);
	m_FrequenzbereichFFT=0;
	m_Redraw=FALSE;
	m_ZoomV=32;
	IniFileRead();
	m_counter=HEIGHT_KLAVIER+2+m_Liniendicke;

	m_ScreenSize.cx=GetSystemMetrics(SM_CXFULLSCREEN);
	m_ScreenSize.cy=GetSystemMetrics(SM_CYFULLSCREEN);
	m_MemDC.CreateCompatibleDC(GetDC());


	m_ScreenBitmap.CreateCompatibleBitmap(GetDC(), m_ScreenSize.cx, m_ScreenSize.cy); 
	m_MemDC.SelectObject(m_ScreenBitmap);
//	m_MemDC.FillSolidRect(0,0,m_ScreenSize.cx,m_ScreenSize.cy,m_clrBkg);

	CLRPOINT Nullpunkt;
	Nullpunkt.x=0;
	Nullpunkt.y=0;
	Nullpunkt.color=RGB(0,0,0);
	m_ViewPtFreqArray.Add(Nullpunkt);


	
	//Palette erstellen
	if (!CreatePalette()) 
		AfxMessageBox(E_SONOVIEW_CREATEPALETTE);
	if (!CreatePaletteWhite()) 
		AfxMessageBox(E_SONOVIEW_CREATEPALETTE);

/*
	// Convert client coordinates to .01-mm units. 
	rect.left = (rect.left * iWidthMM * 100)/iWidthPels; 
	rect.top = (rect.top * iHeightMM * 100)/iHeightPels; 
	rect.right = (rect.right * iWidthMM * 100)/iWidthPels; 
	rect.bottom = (rect.bottom * iHeightMM * 100)/iHeightPels; 

	m_MetaDC.CreateEnhanced(pRefDC,NULL,rect,"SOUNDS Soundanalyse\0Sonogramm Abbild\0\0");
	m_pMetaFile=m_MetaDC.CloseEnhanced();
*/

	m_oldwidth=0;
	m_oldheight=0;

	
	//Dialogparameter initialisieren
	m_pParentFFTFrame=(CFFTFrame*)GetParentFrame();
	m_pBottumCutCtrl=(CSpinButtonCtrl*)m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_BOTTOMCUT_SPIN);
 	m_pBottumCutCtrl->SetRange(1,999);
	m_pBottumCutCtrl->SetBuddy(m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_BOTTOMCUT_FELD));
	m_pBottumCutCtrl->SetPos(m_BottomCutHz);

	m_pTopCutCtrl=(CSpinButtonCtrl*)m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_TOPCUT_SPIN);
	m_pTopCutCtrl->SetRange(1,100);
	m_pTopCutCtrl->SetBuddy(m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_TOPCUT_FELD));
	m_pTopCutCtrl->SetPos(m_TopCutkHz);

	m_pLiniendickeCtrl=(CSpinButtonCtrl*)m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_LINIENDICKE_SPIN);
	m_pLiniendickeCtrl->SetRange(1,9);
	m_pLiniendickeCtrl->SetBuddy(m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_LINIENDICKE_FELD));
	m_pLiniendickeCtrl->SetPos(m_Liniendicke);

	m_pRadioAmpLin=(CButton*)m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_SCALE_AMP_LIN);
	m_pRadioAmpLog=(CButton*)m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_SCALE_AMP_LOG);
	if (m_bIsAmpLinear) 
	{
		m_pRadioAmpLin->SetCheck(1);
		m_pRadioAmpLog->SetCheck(0);
	}
	else
	{
		m_pRadioAmpLin->SetCheck(0);
		m_pRadioAmpLog->SetCheck(1);
	}

	m_pRadioFreqLin=(CButton*)m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_SCALE_FREQ_LIN);
	m_pRadioFreqLog=(CButton*)m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_SCALE_FREQ_LOG);
	if (m_bIsFreqLinear) 
	{
		m_pRadioFreqLin->SetCheck(1);
		m_pRadioFreqLog->SetCheck(0);
	}
	else
	{
		m_pRadioFreqLin->SetCheck(0);
		m_pRadioFreqLog->SetCheck(1);
	}

	m_pRadioFarbig=(CButton*)m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_PALETTE_FARBIG);
	m_pRadioBW=(CButton*)m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_PALETTE_FARBIG2);
	if (m_bIsFarbig1) 
	{
		OnPaletteFarbig();
		m_pRadioFarbig->SetCheck(1);
		m_pRadioBW->SetCheck(0);
	}
	else
	{
		OnPaletteFarbig2();
		m_pRadioFarbig->SetCheck(0);
		m_pRadioBW->SetCheck(1);
	}

	m_pFFTPointsFeld=(CEdit*)m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_FFTPOINTS_FELD);
	m_pFFTPointsFeld->EnableWindow(FALSE);
	char buffer[20];
	CString TextFFTSize;
	TextFFTSize=itoa( *theApp.GetIniDokumentFFTPoints(), buffer, 10 );
	m_pFFTPointsFeld->SetWindowText(TextFFTSize);

	CSliderCtrl* SliderZoomVCtrl=(CSliderCtrl*)m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_SLIDER_ZOOM_V);
	SliderZoomVCtrl->SetTicFreq( 1 );    // Send TBM_SETTICFREQ
	SliderZoomVCtrl->SetLineSize( 1 );       // Send TBM_SETLINESIZE
	SliderZoomVCtrl->SetPageSize( 1 );       // Send TBM_SETPAGESIZE
	SliderZoomVCtrl->SetRange( 0, 10, TRUE );
	SliderZoomVCtrl->SetPos(5);
	m_bEraseClientScreen=TRUE;

	CButton* pCheckButton=(CButton*)m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_CHECK_SMOOTH);
	if (m_bIsSmooth) 
		pCheckButton->SetCheck(TRUE);
	else
		pCheckButton->SetCheck(FALSE);

	Invalidate(TRUE);
	
	CDC* pDC=GetDC();
	ZeichneTasten(pDC);
	ZeichneFrequenzen(pDC);

}

/////////////////////////////////////////////////////////////////////////////
// Zeichnung CSonoView 

void CSonoView::OnDraw(CDC* pDC)
{
	CRect rect;
	GetClientRect(rect);
	CSoundsDoc* pDoc=GetDocument();

	if (m_bEraseClientScreen) 
	{
		m_MemDC.FillSolidRect(0,0,m_ScreenSize.cx,m_ScreenSize.cy,m_clrBkg);
		m_counter=1+m_Liniendicke;
		m_bEraseClientScreen=FALSE;
	}

	if (g_bIsCapturing||g_bIsPlayingWave) 
	{
		CArray<DOUBLE,DOUBLE>* pFrequenzenArray=pDoc->GetFrequenzenArray();
		DrawSpektroLine(&m_MemDC,pFrequenzenArray);
	  pDC->StretchBlt(HEIGHT_KLAVIER, 0, rect.right-HEIGHT_FREQ_RULER-HEIGHT_KLAVIER,rect.bottom, &m_MemDC,
										0, 0, m_ScreenSize.cx, m_ScreenSize.cy, SRCCOPY); 
	}
	else	//hier wird kein Wave gespielt, kein Mikrofon abgehrt 
				//also nur Memorykontext ausgeben und Umgebung anpassen
	{
		pDC->StretchBlt(HEIGHT_KLAVIER, 0, rect.right-HEIGHT_FREQ_RULER-HEIGHT_KLAVIER,rect.bottom, &m_MemDC, 
								0, 0, m_ScreenSize.cx, m_ScreenSize.cy, SRCCOPY); 
		m_Redraw=FALSE;
	}
	ZeichneTasten(pDC);
	ZeichneFrequenzen(pDC);
}

void CSonoView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	CDC* pDC=GetDC();
	ZeichneTasten(pDC);
	ZeichneFrequenzen(pDC);
}

void CSonoView::DrawCompleteSono()
{
	BeginWaitCursor();
	g_bIsBusy=TRUE;

	//Memorykontext lschen
	m_MemDC.FillSolidRect(0,0,m_ScreenSize.cx,m_ScreenSize.cy,m_clrBkg);

	//zu untersuchende Zeitreihe einlesen
	CSoundsDoc* pDoc=GetDocument();
	CArray<DOUBLE,DOUBLE>* pZeitreiheArray=pDoc->GetZeitreiheArray();
	DOUBLE MaxZeitZeitreihe=pZeitreiheArray->GetSize();
	INT FFTPoints=*pDoc->GetFFTPoints();
	CDC* pDC=GetDC();
	if (MaxZeitZeitreihe>FFTPoints)
	{
		m_strHint=_T("");
		CRect rect;
		GetClientRect(rect);
		//Aus Zeitreihe wandernde Ausschnitte analysieren und zeichnen
		//Schrittweite so einstellen, das mglichst 1 FFT pro Linienbreite berechnet wird
		INT Schrittweite=MAX((INT)((MaxZeitZeitreihe-FFTPoints)/(m_ScreenSize.cx-HEIGHT_KLAVIER)*m_Liniendicke),1);
		
		//Linienzhler zurcksetzen
		m_counter=2+m_Liniendicke;
		
		//FFTs berechnen und in Memorykontext zeichnen
		for (INT j=0;j<=(INT)(MaxZeitZeitreihe-FFTPoints); j+=Schrittweite)
		{
			CArray<DOUBLE,DOUBLE> FrequenzenArray;
			pDoc->DoFFTAtPoint(pZeitreiheArray,j,FFTPoints,&FrequenzenArray);
			DrawSpektroLine(&m_MemDC,&FrequenzenArray);
			//Zwischendurch das Bild aktualisieren
			if (j%(Schrittweite*10)==0)
				pDC->StretchBlt(HEIGHT_KLAVIER, 0, rect.right-HEIGHT_FREQ_RULER-HEIGHT_KLAVIER,rect.bottom, &m_MemDC,
													0, 0, m_ScreenSize.cx, m_ScreenSize.cy, SRCCOPY); 

			if (!g_bIsCapturing) 
			{
				//Abbruch mit ESC abfangen
				MSG msg;
				PeekMessage( &msg, NULL, 0, 0, PM_REMOVE );
        if (msg.message == WM_KEYDOWN && msg.wParam == VK_ESCAPE) 
				{
					g_bIsBreak=TRUE;
	        TranslateMessage(&msg); 
		      DispatchMessage(&msg); 
				}
				if (g_bIsBreak) break;
			}
			//ber den Rand hinausmalen abfangen
			if (m_counter>m_ScreenSize.cx-m_Liniendicke) break;
		}
	}
	else
	{
		if (*theApp.GetIniAllgemeinGrundschulprofil()) 
			m_strHint.LoadString(IDS_NEEDMOREPOINTS_SIMPLE);
		else
			m_strHint.LoadString(IDS_NEEDMOREPOINTS);
	}
	ReleaseDC(pDC);

	g_bIsBreak=FALSE;
	g_bIsBusy=FALSE;
	EndWaitCursor();
}

void CSonoView::DrawSpektroLine(CDC* pDC,CArray<DOUBLE,DOUBLE>* pFrequenzenArray)
{
	CSoundsDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	CRect rect;
	GetClientRect(rect);

	if ((m_counter>m_ScreenSize.cx-m_Liniendicke)||(m_counter==1+m_Liniendicke))
	{
		m_counter=2+m_Liniendicke;
		pDC->FillSolidRect(HEIGHT_KLAVIER+2,0,m_ScreenSize.cx+HEIGHT_FREQ_RULER,m_ScreenSize.cy,m_clrBkg);
	}

	// Arbeitslinie in Lila zeichnen
	CPen Stift1,Stift2;
  Stift1.CreatePen(PS_SOLID,m_Liniendicke,RGB(204,55,114));
  pDC->SelectObject(&Stift1);
	pDC->MoveTo(m_counter+m_Liniendicke*2,0);
	pDC->LineTo(m_counter+m_Liniendicke*2,m_ScreenSize.cy);

	//Palette laden
	if(m_bIsFarbig1)
		pDC->SelectPalette(&m_Pal1, FALSE);
	else
		pDC->SelectPalette(&m_Pal2, FALSE);
	pDC->RealizePalette();

	//Maximalamplitude aus Bitrate bestimmen
	DOUBLE MaxAmplitude=0;
	if (m_bIsAmpLinear)
			MaxAmplitude=ldexp(1,*pDoc->GetBitRate())/m_ZoomV;
	else
			MaxAmplitude=log(ldexp(1,*pDoc->GetBitRate())/m_ZoomV);

	INT* pSampleRate=pDoc->GetSampleRate();
	DOUBLE MaxZeit=pFrequenzenArray->GetSize();
	DOUBLE FaktorY=0;
	LONG LastY=m_ScreenSize.cy;
	DOUBLE LastWert=0;		
		//Frequenzarray auf Fenster skalieren
		if (MaxZeit>2)
		{
			m_ViewPtFreqArray.RemoveAll();
//			m_ViewPtFreqArray.SetSize((INT)(MaxZeit+10));
			CLRPOINT Messpunkt;
		  FaktorY=m_ScreenSize.cy/MaxZeit;
			
			int i=0;
			INT FrequenzhoeheSonoAnsicht=m_TopCutkHz*1000;//*pSampleRate/2;
			DOUBLE BildProzentVonBottomCutHz = m_bIsFreqLinear?\
																				 m_BottomCutHz/(DOUBLE)FrequenzhoeheSonoAnsicht:\
																				 log(m_BottomCutHz)/log(FrequenzhoeheSonoAnsicht);
//			FILE *FileStream;
//			FileStream=fopen("e:\\Studium\\Examen\\C++\\Color.txt","w");
			while (i<(INT)(MaxZeit))
			{
				INT FrequenzLinear=(INT)(i*(*pSampleRate/2/MaxZeit));
//				INT FrequenzLinear=(INT)(i*(FrequenzhoeheSonoAnsicht/MaxZeit));
				//Frequenzen unter m_BottomCutHz ausschneiden wegen Darstellungsverlust
				//die ersten 10 Hz entsprechen ca. 1/5 des Bildraums in der Oktavdarstellung
				if (FrequenzLinear>m_BottomCutHz) 
				{
					//Frequenzen von m_BottomCutHz bis Frequenzbreite (22khz, 44kHz usw) auf ClientRect skalieren
					DOUBLE ProzentwertKlein=m_bIsFreqLinear?\
																	FrequenzLinear/(DOUBLE)FrequenzhoeheSonoAnsicht-BildProzentVonBottomCutHz:\
																	log(FrequenzLinear)/log(FrequenzhoeheSonoAnsicht)-BildProzentVonBottomCutHz;
					DOUBLE ProzentwertSkaliertAuf100=ProzentwertKlein/(1-BildProzentVonBottomCutHz);
					Messpunkt.x=m_counter;
		      Messpunkt.y=(LONG)(m_ScreenSize.cy*(1-ProzentwertSkaliertAuf100));


				
					if (Messpunkt.y<LastY)
					{
						DOUBLE Intensitaet=(pFrequenzenArray->GetAt(i));
						DOUBLE Wert=0;
						if (m_bIsAmpLinear)
							Wert=Intensitaet<MaxAmplitude?Intensitaet/MaxAmplitude:1;
						else
							Wert=log(Intensitaet)<MaxAmplitude?log(Intensitaet)/MaxAmplitude:1;
						if (Messpunkt.y<LastY-2&&m_bIsSmooth&&i>0) 
						{
							CLRPOINT NewMesspunkt;
							NewMesspunkt.x=Messpunkt.x;
							NewMesspunkt.y=LastY-1;
							DOUBLE Steigung=(Wert-LastWert)/(LastY-Messpunkt.y);
							DOUBLE NewWert=LastWert+Steigung;
							NewMesspunkt.color=PALETTEINDEX((INT)(m_bIsAmpLinear?NewWert*1279:pow(NewWert,3)*1279));
//							NewMesspunkt.color=PALETTEINDEX((INT)(NewWert*1279));
								m_ViewPtFreqArray.Add(NewMesspunkt);
							int z=2;
							while (NewMesspunkt.y>Messpunkt.y) 
							{
								NewMesspunkt.x=Messpunkt.x;
								NewWert=LastWert+z*Steigung;
								NewMesspunkt.color=PALETTEINDEX((INT)(m_bIsAmpLinear?NewWert*1279:pow(NewWert,3)*1279));
								NewMesspunkt.y=LastY-z;
								m_ViewPtFreqArray.Add(NewMesspunkt);
								z++;
							}
						}
						Messpunkt.color=PALETTEINDEX((INT)(m_bIsAmpLinear?Wert*1279:pow(Wert,3)*1279));
// 					fprintf (FileStream,"%5d %+12.6f %+12.6f\n", i, Wert,(DOUBLE)Messpunkt.color);
//						if (Messpunkt.x<rect.right-HEIGHT_FREQ_RULER-m_Liniendicke) 
//						{
							m_ViewPtFreqArray.Add(Messpunkt);
							LastY=Messpunkt.y;
							LastWert=Wert;
//						}
					}
				}
				else
				{
					DOUBLE Intensitaet=(pFrequenzenArray->GetAt(i));
					DOUBLE Wert=0;
					if (m_bIsAmpLinear)
						Wert=Intensitaet<MaxAmplitude?Intensitaet/MaxAmplitude:1;
					else
						Wert=log(Intensitaet)<MaxAmplitude?log(Intensitaet)/MaxAmplitude:1;
					LastWert=Wert;
				}
				i+=1;//Sprung;
			}
// 			fclose(FileStream);
		}

		//Frequenzen zeichnen
		Stift2.CreatePen(PS_SOLID,m_Liniendicke,m_clrBkg);
		if (m_ViewPtFreqArray.GetSize()>1) 
		{
		  pDC->SelectObject(&Stift2);
			pDC->MoveTo(m_counter+m_Liniendicke,0);
			pDC->LineTo(m_counter+m_Liniendicke,m_ScreenSize.cy);
			INT i=0;
			LONG oldheight=m_ScreenSize.cy;
			while (i < (m_ViewPtFreqArray.GetSize()-1))
		  {
			  CLRPOINT NextPunkt=m_ViewPtFreqArray.GetAt(i);
				if (NextPunkt.y<oldheight) 
				{
 					pDC->FillSolidRect(NextPunkt.x,NextPunkt.y,m_Liniendicke,oldheight-NextPunkt.y,NextPunkt.color);
					oldheight=NextPunkt.y;
				}
				i++;
	    }
			m_counter+=m_Liniendicke;
		}
}

void CSonoView::ZeichneFrequenzen(CDC* pDC)
{
	CSoundsDoc* pDoc = GetDocument();
	CRect rect;
	GetClientRect(rect);

	pDC->FillSolidRect(rect.right-HEIGHT_FREQ_RULER,0,rect.right,rect.bottom,m_clrBkg);
	CPen KurvenStift;
	KurvenStift.CreatePen(PS_SOLID,1,RGB(255,0,0));
	pDC->SelectObject(&KurvenStift);

	CAutoFont font("Century Gothic");
	font.SetDC(pDC->GetSafeHdc());
	font.SetHeight(15);
	font.SetFontColor(m_clrText);

	CFont *oldFont=pDC->SelectObject(&font);
	pDC->SetBkMode(TRANSPARENT);
	pDC->SetTextColor(font.GetFontColor());

	pDC->DrawText(m_strHint,rect,DT_TOP|DT_CENTER);

	INT SampleRate=*pDoc->GetSampleRate();

		DWORD StricheProClientRect=m_bIsFreqLinear?(DWORD)((DOUBLE)SampleRate/2000):18;
		INT FrequenzhoeheSonoAnsicht=m_TopCutkHz*1000;

		//Frequenzen unter m_BottomCutHz ausschneiden wegen Darstellungsverlust
		//die ersten 10 Hz entsprechen ca. 1/5 des Bildraums in der Oktavdarstellung
		DOUBLE BildProzentVonBottomCutHz = m_bIsFreqLinear?\
																			 m_BottomCutHz/(DOUBLE)FrequenzhoeheSonoAnsicht:\
																			 log(m_BottomCutHz)/log(FrequenzhoeheSonoAnsicht);
		for (UINT i=0; i<=StricheProClientRect; i++)
		{
			m_FrequenzbereichFFT=FrequenzhoeheSonoAnsicht-m_BottomCutHz;
			INT FrequenzLinear;
			INT Faktor=(INT)(MAX((INT)floor((((DOUBLE)SampleRate/88200+(DOUBLE)m_FrequenzbereichFFT/11000))),1));
			if (m_bIsFreqLinear)
				FrequenzLinear=(INT)(Faktor*1000*i);
			else
				FrequenzLinear=(INT)(ldexp(1.71875,i));

			//Frequenzen von m_BottomCutHz bis m_TopCutkHz auf ClientRect skalieren
			DOUBLE ProzentwertKlein=m_bIsFreqLinear?\
															FrequenzLinear/(DOUBLE)FrequenzhoeheSonoAnsicht-BildProzentVonBottomCutHz:\
															log(FrequenzLinear)/log(FrequenzhoeheSonoAnsicht)-BildProzentVonBottomCutHz;
			DOUBLE ProzentwertSkaliertAuf100=ProzentwertKlein/(1-BildProzentVonBottomCutHz);
			LONG yWert=(LONG)(rect.bottom-ProzentwertSkaliertAuf100*rect.bottom);


			
			if (yWert>=0&&yWert<=rect.bottom) 
			{
				pDC->MoveTo(rect.right-HEIGHT_FREQ_RULER,yWert);
				pDC->LineTo(rect.right-HEIGHT_FREQ_RULER+7,yWert);
			}
			char buffer[20];
			CString AddOn;
			AddOn=" Hz";
			CString Zeit;
			Zeit=itoa( FrequenzLinear, buffer, 10 );
			if (FrequenzLinear>1000) 
			{
				INT FrequenzTausend=FrequenzLinear/1000;
				if (FrequenzTausend>10) 
					Zeit=itoa( FrequenzTausend, buffer, 10 );
				else
				{
					INT Zweite=(FrequenzLinear%1000)/100;
					CString strErste=itoa( FrequenzTausend, buffer, 10 );
					CString strZweite=itoa( Zweite, buffer, 10 );
					Zeit=strErste+","+strZweite;
				}
				AddOn=" kHz";
			}
			CRect TextRect;
			TextRect.top=yWert-7;
			TextRect.bottom=yWert+7;
			TextRect.left=rect.right-HEIGHT_FREQ_RULER+6;
			TextRect.right=rect.right;
			if ((yWert-7)>=0&&(yWert-7)<=rect.bottom) 
				pDC->DrawText(Zeit+AddOn,TextRect,DT_WORDBREAK);
//			DWORD Rasterklein=(DWORD)(Raster);
			LONG Marker1Pos,Marker2Pos,LastyWert;
			Marker2Pos=yWert;
			if (i>0) 
			 Marker1Pos=LastyWert;

			if (i>0) 
			{
				DOUBLE PixelbreiteMarkerabstandKlein=(DOUBLE)(Marker2Pos-Marker1Pos)/12;
				for (INT j=1; j<12; j++ )
				{
					if ((yWert-(LONG)(j*PixelbreiteMarkerabstandKlein))>=0&&\
							(yWert-(LONG)(j*PixelbreiteMarkerabstandKlein))<=rect.bottom) 
					{
						pDC->MoveTo(rect.right-HEIGHT_FREQ_RULER,yWert-(LONG)(j*PixelbreiteMarkerabstandKlein));
						pDC->LineTo(rect.right-HEIGHT_FREQ_RULER+3,yWert-(LONG)(j*PixelbreiteMarkerabstandKlein));
					}
				}
			}
			LastyWert=yWert;
		}


//	}
	pDC->SelectObject(oldFont);
}


void CSonoView::ZeichneTasten(CDC* pDC) 
{
		CRect rect;
		GetClientRect(rect);
		CAutoFont font("Century Gothic");
		font.SetDC(pDC->GetSafeHdc());
		font.SetHeight(16);
		font.SetFontColor(m_clrText);
		CFont *oldFont=pDC->SelectObject(&font);
		pDC->SetBkMode(TRANSPARENT);
		pDC->SetTextColor(font.GetFontColor());
		CPen WeissStift;
	  WeissStift.CreatePen(PS_SOLID,8,RGB(255,255,255));
		CPen SchwarzStift;
	  SchwarzStift.CreatePen(PS_SOLID,1,RGB(0,0,0));
		CPen GrauStift;
	  GrauStift.CreatePen(PS_SOLID,8,RGB(100,100,100));
	  pDC->SelectObject(&WeissStift);
		CBrush BrushWhite;
		BrushWhite.CreateSolidBrush(RGB(255,255,255));
		pDC->SelectObject(&BrushWhite);
		CBrush BrushBlack;
		BrushBlack.CreateSolidBrush(RGB(0,0,0));
		pDC->Rectangle (0,0,HEIGHT_KLAVIER,rect.bottom);
	  pDC->SelectObject(&SchwarzStift);
		pDC->MoveTo(HEIGHT_KLAVIER+3,0);
		pDC->LineTo(HEIGHT_KLAVIER+3,rect.bottom);
		LONG LastyWert,Abstand;

		INT FrequenzhoeheSonoAnsicht=m_TopCutkHz*1000;
		DOUBLE BildProzentVonBottomCutHz = m_bIsFreqLinear?\
																			 m_BottomCutHz/(DOUBLE)FrequenzhoeheSonoAnsicht:\
																			 log(m_BottomCutHz)/log(FrequenzhoeheSonoAnsicht);
		for (UINT i=0; i<=88; i++)
		{
			DOUBLE Cent=1.00057778951;//ldexp(1,(DOUBLE)1/(DOUBLE)1200);
			DOUBLE FrequenzLinear=pow(Cent,(i*100))*27.5;
			DOUBLE ProzentwertKlein=m_bIsFreqLinear?\
															FrequenzLinear/(DOUBLE)FrequenzhoeheSonoAnsicht-BildProzentVonBottomCutHz:\
															log(FrequenzLinear)/log(FrequenzhoeheSonoAnsicht)-BildProzentVonBottomCutHz;
			DOUBLE ProzentwertSkaliertAuf100=ProzentwertKlein/(1-BildProzentVonBottomCutHz);
			m_FrequenzbereichFFT=FrequenzhoeheSonoAnsicht-m_BottomCutHz;
			LONG yWert=(LONG)(rect.bottom-ProzentwertSkaliertAuf100*rect.bottom);
			if (i>0) Abstand=yWert-LastyWert;
			else Abstand=8;
			pDC->SelectObject(&SchwarzStift);
			if ((i==2)||(i==5)||(i==7)||(i==10)||(i==12)||(i==14)||(i==17)||(i==19)||(i==22)||(i==24)\
					||(i==26)||(i==29)||(i==31)||(i==34)||(i==36)||(i==38)||(i==41)||(i==43)||(i==46)||(i==48)\
					||(i==50)||(i==53)||(i==55)||(i==58)||(i==60)||(i==62)||(i==65)||(i==67)||(i==70)\
					||(i==72)||(i==74)||(i==77)||(i==79)||(i==82)||(i==84)||(i==86))
			{
				pDC->SelectObject(&BrushBlack);
				pDC->SelectObject(&SchwarzStift);
				pDC->MoveTo(HEIGHT_KLAVIER-8,LastyWert+Abstand/2-Abstand/2);
				pDC->LineTo(HEIGHT_KLAVIER+3,LastyWert+Abstand/2-Abstand/2);
/*				pDC->LineTo(xWert,HEIGHT_KLAVIER-5);
				pDC->LineTo(xWert,0);
*/
				pDC->Rectangle (0,LastyWert-Abstand/2,HEIGHT_KLAVIER-8,yWert-Abstand/2);
			}
			if ((i==3)||(i==8)||(i==15)||(i==20)||(i==27)||(i==32)||(i==39)||(i==44)||(i==51)\
					||(i==56)||(i==63)||(i==68)||(i==75)||(i==80)||(i==87))			//(i=0)
			{
				pDC->MoveTo(0,yWert-Abstand/2);
				pDC->LineTo(HEIGHT_KLAVIER+3,yWert-Abstand/2);
			}
			if (i==88)
			{
				pDC->MoveTo(0,yWert);
				pDC->LineTo(HEIGHT_KLAVIER+3,yWert);
// 				pDC->TextOut(HEIGHT_KLAVIER-18,yWert+20,"Tonumfang Klavier");
			}
			if (i==39)
				pDC->TextOut(HEIGHT_KLAVIER-10,yWert-10,"c");
			LastyWert=yWert;			
		}
		pDC->SelectObject(oldFont);
//	}
}

void CSonoView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	m_Redraw=TRUE;
	Invalidate(TRUE);
	if (lHint==50||lHint==12)
	{
		if (g_bIsPlayingWave||g_bIsCapturing)
		{
		CSoundsDoc* pDoc=GetDocument();
		INT NumFFTPoints=*pDoc->GetFFTPoints();
		char buffer[20];
		CString Pts=itoa( NumFFTPoints, buffer, 10 );
		CEdit* pEditNumFFTPoints=(CEdit*)m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_FFTPOINTS_FELD);
		pEditNumFFTPoints->SetWindowText(Pts);
		CRect rect;
		GetClientRect(rect);
		rect.right=HEIGHT_KLAVIER+2;
		ValidateRect(rect);
		GetClientRect(rect);
		rect.left=rect.right-HEIGHT_FREQ_RULER+m_Liniendicke;
		ValidateRect(rect);
		GetClientRect(rect);
		rect.right=m_counter;
//	ValidateRect(rect);
		}
	}
	if (g_bIsPutPoints&&!g_bIsBreak) DrawCompleteSono();
	if (lHint==11)
	{
		m_MemDC.FillSolidRect(HEIGHT_KLAVIER,0,m_ScreenSize.cx+HEIGHT_FREQ_RULER,m_ScreenSize.cy,m_clrBkg);
	}
}

BOOL CSonoView::OnEraseBkgnd(CDC* pDC) 
{
	CRect rect;
//	GetClientRect(rect);
//	rect.left=HEIGHT_KLAVIER;
//	rect.right=HEIGHT_FREQ_RULER;
//	pDC->FillSolidRect(rect,m_clrBkg);

//	return CView::OnEraseBkgnd(pDC);
	return FALSE;
}

void CSonoView::OnLButtonUp(UINT nFlags, CPoint point) 
{
/*
	if (!g_bIsPlayingWave)//||!g_bIsCapturing) 
	  m_Gen.StopGen();
*/
	CDC* pDC=GetDC();
	CRect rect;
	GetClientRect(rect);
	CPen pe(PS_NULL,0,RGB(255,255,255));
	CPen* oldp =pDC->SelectObject (&pe);
//		pDC->SetROP2(R2_MASKPEN);
	CRect Region;
	Region.left=HEIGHT_KLAVIER;
	Region.top=m_LButtonDownMousePoint.y+1;
	Region.bottom=m_LButtonDownMousePoint.y;
	Region.right=m_LButtonDownMousePoint.x;
	pDC->InvertRect (&Region);
	pDC->SelectObject (oldp);

	CView::OnLButtonUp(nFlags, point);
}

void CSonoView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	//	CSoundsDoc* pDoc = GetDocument();
//	INT* pSampleRate=pDoc->GetSampleRate();
//
//	if (*pSampleRate!=0) 
//	{
		CDC* pDC=GetDC();
		CRect rect;
		GetClientRect(rect);
/*

		CAutoFont font("Century Gothic");
		font.SetDC(pDC->GetSafeHdc());
		font.SetHeight(16);
		font.SetFontColor(m_clrText);
		CFont *oldFont=pDC->SelectObject(&font);
		pDC->SetBkMode(TRANSPARENT);
		pDC->SetTextColor(font.GetFontColor());
//		CPen Stift;
//	  Stift.CreatePen(PS_SOLID,1,m_clrText);
//	  pDC->SelectObject(&Stift);
*/

/*
		CPen pe(PS_NULL,0,RGB(255,255,255));
		CPen* oldp =pDC->SelectObject (&pe);
		CRect Region;
		Region.left=HEIGHT_KLAVIER;
		Region.top=point.y+1;
		Region.bottom=point.y;
		Region.right=m_counter;
		m_LButtonDownMousePoint=point;
		m_LButtonDownMousePoint.x=m_counter;
 		pDC->InvertRect (&Region);
		pDC->SelectObject (oldp);
*/

		
//		pDC->MoveTo(0,point.y);
//		pDC->LineTo(rect.right,point.y);

		INT FrequenzhoeheSonoAnsicht=m_TopCutkHz*1000;//*pSampleRate/2;
		DOUBLE BildProzentVonBottomCutHz = m_bIsFreqLinear?\
																			 m_BottomCutHz/(DOUBLE)FrequenzhoeheSonoAnsicht:\
																			 log(m_BottomCutHz)/log(FrequenzhoeheSonoAnsicht);
		DOUBLE ProzentwertSkaliertAuf100=(DOUBLE)((rect.bottom-point.y)/(FLOAT)rect.bottom);
		DOUBLE ProzentwertKlein=ProzentwertSkaliertAuf100*(1-BildProzentVonBottomCutHz);

		DOUBLE Frequenz=m_bIsFreqLinear?\
										(ProzentwertKlein+BildProzentVonBottomCutHz)*FrequenzhoeheSonoAnsicht:
										exp((ProzentwertKlein+BildProzentVonBottomCutHz)*log(FrequenzhoeheSonoAnsicht));
//		char buffer[20];
//		CString AddOn;
//		AddOn=" Hz";
//		CString strFrequenz=gcvt( Frequenz, 5, buffer);

//		pDC->TextOut(point.x+5,point.y-15,strFrequenz+AddOn);
//		pDC->SelectObject(oldFont);

		//Ton ausgeben, wenn auf Klaviertaste gedrckt
		//Ton ausgeben, wenn auf Klaviertaste gedrckt
		if (point.x<=HEIGHT_KLAVIER) 
		{
//			if (g_bIsPlayingWave) 
				Beep((INT)Frequenz,400);
//			else
//			{
//				m_Gen.FillBuf((INT)Frequenz,1,0,1);
//				m_Gen.GenFreq();
//			}
		}
		char buffer[20];
		CString strFrequenz=gcvt( Frequenz, 5, buffer);

		m_pParentFFTFrame->m_wndDlgBar.SetDlgItemText(IDC_FREQUENZ,strFrequenz);

		
		//	}
	CView::OnLButtonDown(nFlags, point);
}

void CSonoView::OnMouseMove(UINT nFlags, CPoint point) 
{
		CRect rect;
		GetClientRect(rect);
		INT FrequenzhoeheSonoAnsicht=m_TopCutkHz*1000;//*pSampleRate/2;
		DOUBLE BildProzentVonBottomCutHz = m_bIsFreqLinear?\
																			 m_BottomCutHz/(DOUBLE)FrequenzhoeheSonoAnsicht:\
																			 log(m_BottomCutHz)/log(FrequenzhoeheSonoAnsicht);
		DOUBLE ProzentwertSkaliertAuf100=(DOUBLE)((rect.bottom-point.y)/(FLOAT)rect.bottom);
		DOUBLE ProzentwertKlein=ProzentwertSkaliertAuf100*(1-BildProzentVonBottomCutHz);

		DOUBLE Frequenz=m_bIsFreqLinear?\
										(ProzentwertKlein+BildProzentVonBottomCutHz)*FrequenzhoeheSonoAnsicht:
										exp((ProzentwertKlein+BildProzentVonBottomCutHz)*log(FrequenzhoeheSonoAnsicht));
	
		char buffer[20];
		CString strFrequenz=gcvt( Frequenz, 5, buffer);
		m_pParentFFTFrame->m_wndDlgBar.SetDlgItemText(IDC_FREQUENZ,strFrequenz);

		if (nFlags==MK_LBUTTON&&point.x<=HEIGHT_KLAVIER) 
		{
			Beep((INT)Frequenz,50);
//			m_Gen.FillBuf((INT)Frequenz,1,0,1);
//			m_Gen.GenFreq();
		}

		CView::OnMouseMove(nFlags, point);
}


COLORREF CSonoView::CreatePaletteCOLORREF(int Index)
{
	const int nNumColors = 1280;
	BYTE R,G,B;
	if (Index>= 0&&Index < 256)
	{
		R = (BYTE)(0);
		G = (BYTE)(0);
		B = (BYTE)(Index);
	}
	else if (Index>= 256&&Index < 512)
	{
		R = (BYTE)(0);
		G = (BYTE)(Index-256);
		B = (BYTE)(255);
	}
	else if (Index>= 512&&Index < 768)
	{
		R = (BYTE)(0);
		G = (BYTE)(255);
		B = (BYTE)(255-Index-512);
	}
	else if (Index>= 768&&Index < 1024)
	{
		R = (BYTE)(Index-768);
		G = (BYTE)(255);
		B = (BYTE)(0);
	}
	else if (Index>= 1024&&Index < 1280)
	{
		R = (BYTE)(255);
		G = (BYTE)(255-Index-1024);
		B = (BYTE)(0);
	}
	else
	{
		R = (BYTE)(255);
		G = (BYTE)(255);
		B = (BYTE)(255);
	}
	return RGB(R,G,B);
}

BOOL CSonoView::CreatePalette()
{
	const int nNumColors = 1280;
	BOOL bSucc=FALSE;
	LPLOGPALETTE lpPal = (LPLOGPALETTE)new BYTE[sizeof(LOGPALETTE) + sizeof(PALETTEENTRY) *	nNumColors];
	
	if (lpPal != NULL)
	{
		lpPal->palVersion = 0x300;
		lpPal->palNumEntries = nNumColors;
		
		PALETTEENTRY *ppe = lpPal->palPalEntry;
		
		for (int nIndex = 0; nIndex < 256; nIndex++)
		{
			ppe->peRed = (BYTE)(0);
			ppe->peGreen = (BYTE)(0);
			ppe->peBlue = (BYTE)(nIndex);
			ppe->peFlags = (BYTE)0;
			ppe++;
		}
		for (nIndex = 0; nIndex < 256; nIndex++)
		{
			ppe->peRed = (BYTE)(0);
			ppe->peGreen = (BYTE)(nIndex);
			ppe->peBlue = (BYTE)(255);
			ppe->peFlags = (BYTE)0;
			ppe++;
		}
		for (nIndex = 0; nIndex < 256; nIndex++)
		{
			ppe->peRed = (BYTE)(0);
			ppe->peGreen = (BYTE)(255);
			ppe->peBlue = (BYTE)(255-nIndex);
			ppe->peFlags = (BYTE)0;
			ppe++;
		}
		for (nIndex = 0; nIndex < 256; nIndex++)
		{
			ppe->peRed = (BYTE)(nIndex);
			ppe->peGreen = (BYTE)(255);
			ppe->peBlue = (BYTE)(0);
			ppe->peFlags = (BYTE)0;
			ppe++;
		}
		for (nIndex = 0; nIndex < 256; nIndex++)
		{
			ppe->peRed = (BYTE)(255);
			ppe->peGreen = (BYTE)(255-nIndex);
			ppe->peBlue = (BYTE)(0);
			ppe->peFlags = (BYTE)0;
			ppe++;
		}
		
		bSucc = m_Pal1.CreatePalette(lpPal);
		
		delete [](PBYTE)lpPal;
	}
	
	return bSucc;
}

BOOL CSonoView::CreatePaletteWhite()
{
	const int nNumColors = 1280;
	BOOL bSucc=FALSE;
	LPLOGPALETTE lpPal = (LPLOGPALETTE)new BYTE[sizeof(LOGPALETTE) + sizeof(PALETTEENTRY) *	nNumColors];
	
	if (lpPal != NULL)
	{
		lpPal->palVersion = 0x300;
		lpPal->palNumEntries = nNumColors;
		
		PALETTEENTRY *ppe = lpPal->palPalEntry;
		
		for (int nIndex = 0; nIndex < 256; nIndex++)
		{
			ppe->peRed = (BYTE)(255-nIndex);
			ppe->peGreen = (BYTE)(255);
			ppe->peBlue = (BYTE)(255);
			ppe->peFlags = (BYTE)0;
			ppe++;
		}
		for (nIndex = 0; nIndex < 256; nIndex++)
		{
			ppe->peRed = (BYTE)(0);
			ppe->peGreen = (BYTE)(255-nIndex);
			ppe->peBlue = (BYTE)(255);
			ppe->peFlags = (BYTE)0;
			ppe++;
		}
		for (nIndex = 0; nIndex < 256; nIndex++)
		{
			ppe->peRed = (BYTE)(0);
			ppe->peGreen = (BYTE)(nIndex);
			ppe->peBlue = (BYTE)(255-nIndex);
			ppe->peFlags = (BYTE)0;
			ppe++;
		}
		for (nIndex = 0; nIndex < 256; nIndex++)
		{
			ppe->peRed = (BYTE)(nIndex);
			ppe->peGreen = (BYTE)(255);
			ppe->peBlue = (BYTE)(0);
			ppe->peFlags = (BYTE)0;
			ppe++;
		}
		for (nIndex = 0; nIndex < 256; nIndex++)
		{
			ppe->peRed = (BYTE)(255);
			ppe->peGreen = (BYTE)(255-nIndex);
			ppe->peBlue = (BYTE)(0);
			ppe->peFlags = (BYTE)0;
			ppe++;
		}
		
		bSucc = m_Pal2.CreatePalette(lpPal);
		
		delete [](PBYTE)lpPal;
	}
	
	return bSucc;
}

void CSonoView::OnSetFocus(CWnd* pOldWnd) 
{
	CView::OnSetFocus(pOldWnd);
	
	m_Redraw=TRUE;	
	
}

void CSonoView::OnPaletteFarbig() 
{
	m_bIsFarbig1=TRUE;
//	m_bEraseClientScreen=TRUE;
	m_clrBkg=RGB(0,0,0);
	m_clrText=RGB(255,255,255);
	Invalidate(TRUE);
}

void CSonoView::OnPaletteFarbig2() 
{
	m_bIsFarbig1=FALSE;
//	m_bEraseClientScreen=TRUE;
	m_clrBkg=RGB(255,255,255);
	m_clrText=RGB(0,0,0);
	Invalidate(TRUE);
}


void CSonoView::OnScaleAmpLin() 
{
	m_bIsAmpLinear=TRUE;
//	m_bEraseClientScreen=TRUE;
	Invalidate(TRUE);
}

void CSonoView::OnScaleAmpLog() 
{
	m_bIsAmpLinear=FALSE;
//	m_bEraseClientScreen=TRUE;
	Invalidate(TRUE);
}

void CSonoView::OnScaleFreqLin() 
{
	m_bIsFreqLinear=TRUE;
	m_bEraseClientScreen=TRUE;
	Invalidate(TRUE);
}

void CSonoView::OnScaleFreqLog() 
{
	m_bIsFreqLinear=FALSE;
	m_bEraseClientScreen=TRUE;
	Invalidate(TRUE);
}

void CSonoView::OnReleasedcaptureSliderZoomV(NMHDR* pNMHDR, LRESULT* pResult) 
{
	CSliderCtrl* SliderZoomVCtrl=(CSliderCtrl*)m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_SLIDER_ZOOM_V);
	INT Stelle=SliderZoomVCtrl->GetPos();
	*pResult = 0;
	m_ZoomV=(INT)(Stelle*4+1);//(1025-ldexp(1,Stelle));
	m_bEraseClientScreen=TRUE;
	Invalidate(TRUE);
}

void CSonoView::OnChangeTopcutFeld() 
{
	m_TopCutkHz=m_pParentFFTFrame->m_wndDlgBar.GetDlgItemInt(IDC_TOPCUT_FELD);
	m_bEraseClientScreen=TRUE;
	Invalidate(TRUE);
}

void CSonoView::OnChangeBottomcutFeld() 
{
	m_BottomCutHz=m_pParentFFTFrame->m_wndDlgBar.GetDlgItemInt(IDC_BOTTOMCUT_FELD);
	m_bEraseClientScreen=TRUE;
	Invalidate(TRUE);
}

void CSonoView::OnChangeLiniendickeFeld() 
{
	m_Liniendicke=m_pParentFFTFrame->m_wndDlgBar.GetDlgItemInt(IDC_LINIENDICKE_FELD);
	m_bEraseClientScreen=TRUE;
	Invalidate(TRUE);
}

void CSonoView::OnCheckSmooth() 
{
	CButton* pCheckButton=(CButton*)m_pParentFFTFrame->m_wndDlgBar.GetDlgItem(IDC_CHECK_SMOOTH);
	m_bIsSmooth=pCheckButton->GetCheck();
}

void CSonoView::OnEditClearall() 
{
	m_bEraseClientScreen=TRUE;
	CSoundsDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	pDoc->DeleteContents();
	Invalidate(TRUE);
	pDoc->UpdateAllViews(NULL,50);
}

CRect CSonoView::DetermineRect(CDC* RefDC)
{
	// Determine the picture frame dimensions. 
	// iWidthMM is the display width in millimeters. 
	// iHeightMM is the display height in millimeters. 
	// iWidthPels is the display width in pixels. 
	// iHeightPels is the display height in pixels 
 
	INT iWidthMM = GetDeviceCaps(RefDC->GetSafeHdc(), HORZSIZE); 
	INT iHeightMM = GetDeviceCaps(RefDC->GetSafeHdc(), VERTSIZE); 
	INT iWidthPels = GetDeviceCaps(RefDC->GetSafeHdc(), HORZRES); 
	INT iHeightPels = GetDeviceCaps(RefDC->GetSafeHdc(), VERTRES); 
 
	// Retrieve the coordinates of the client rectangle, in pixels. 
	CRect rect;
	GetClientRect(rect); 
 
	// Convert client coordinates to .01-mm units. 
	// Use iWidthMM, iWidthPels, iHeightMM, and 
	// iHeightPels to determine the number of 
	// .01-millimeter units per pixel in the x- 
	//  and y-directions. 
 
	rect.left = (rect.left * iWidthMM * 100)/iWidthPels; 
	rect.top = (rect.top * iHeightMM * 100)/iHeightPels; 
	rect.right = (rect.right * iWidthMM * 100)/iWidthPels; 
	rect.bottom = (rect.bottom * iHeightMM * 100)/iHeightPels; 

	return rect;
}

void CSonoView::OnSave() 
{
	// Obtain a handle to a reference device context. 
	CDC* RefDC = GetDC(); 
 
	CRect rect=DetermineRect(RefDC);
	
	// Load the filename Dialog
	CFileDialog FotoFFTFileDlg(FALSE,NULL,NULL,OFN_OVERWRITEPROMPT|OFN_NOREADONLYRETURN|OFN_ENABLESIZING,\
              "WMF - Enhanced Metafile|*.wmf|");
  FotoFFTFileDlg.m_ofn.nFilterIndex=1;
	FotoFFTFileDlg.m_ofn.lpstrDefExt=".wmf";
	FotoFFTFileDlg.m_ofn.lpstrTitle="Speichert Sonogramm...";
  if (FotoFFTFileDlg.DoModal()==IDOK) 
	{
		// Create the metafile device context. 
		CString Pfad=FotoFFTFileDlg.GetPathName();
		LPTSTR strFileName=Pfad.GetBuffer(MAX_PATH);
		CMetaFileDC MetaFileDC;
		MetaFileDC.CreateEnhanced(RefDC,strFileName,rect,"SOUNDS Soundanalyse\0Sonogramm Abbild\0\0");
	
		OnDraw(&MetaFileDC);
		HENHMETAFILE pMetaFile=MetaFileDC.CloseEnhanced();
		DeleteEnhMetaFile(pMetaFile);
	}
	// Release the reference device context. 
	ReleaseDC(RefDC); 
	
}

void CSonoView::OnEditCopy() 
{
	CDC* RefDC = GetDC(); 
 
	CRect rect=DetermineRect(RefDC);

	CMetaFileDC * pMetaDC = new CMetaFileDC();
	pMetaDC->CreateEnhanced(GetDC(),NULL,rect,"SOUNDS Soundanalyse\0Sonogramm Abbild\0\0");
	//draw meta file
//	m_clrBkg=RGB(255,255,255);
//	m_clrText=RGB(0,0,0);
//	CRect rect;
//	GetClientRect(rect);

//	pMetaDC->BitBlt(0,0,rect.right-HEIGHT_KLAVIER-3,rect.bottom,m_pMemDC,HEIGHT_KLAVIER+3,0,SRCCOPY);
//	pMetaDC->BitBlt(0,0,rect.right,rect.bottom,&m_MemDC,0,0,SRCCOPY);
	OnDraw(pMetaDC);
	//close meta file dc and prepare for clipboard;
	HENHMETAFILE hMF = pMetaDC->CloseEnhanced();

	//copy to clipboard
	// Open the clipboard...
	if(!OpenClipboard()) 
	{
    AfxMessageBox(IDS_OPEN_CLIPBOARD_FAIL);
    return;
	}
	EmptyClipboard();
	::SetClipboardData(CF_ENHMETAFILE,hMF);
	CloseClipboard();
	//DeleteMetaFile(hMF);
	delete pMetaDC;
}

BOOL CSonoView::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	CSoundsDoc* pDoc=GetDocument();
	pDoc->m_help.SetHelpFile(pDoc->m_strHelpFile);
	pDoc->m_help.DisplayTopic("html/Sonogrammdarstellung.html");
	
	return CView::OnHelpInfo(pHelpInfo);
}

