// Splines.cpp: Implementierung der Klasse CSplines.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Splines.h"
#include <stdlib.h>
#include <assert.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CSplines::CSplines()
{
	m_dBeta1=2;
	m_dBeta2=0;
}

CSplines::~CSplines()
{

}


void CSplines::ZeichneBSpline(CDC* pDC, const POINT paPoints [], size_t nSize,COLORREF color)
{
  size_t i;
  PMATRIX pMs, pT;

  assert (paPoints != NULL && nSize > 0);


  if (!InitBetaSpline (&pMs, &pT, m_dBeta1, m_dBeta2)) 
//    AfxMessageBox (E_NOT_ENOUGH_MEMORY);
    AfxMessageBox ("Not enough memory");
	CPen Stift,*pOldPen;
	Stift.CreatePen(PS_SOLID,1,color);
	pOldPen=pDC->SelectObject(&Stift);

  for (i = 0; i < nSize - 3 ; i ++) {
	  double t;

    for (t = 0; t < 1.0; t+=0.1) {
      PMATRIX pResult;
      POINT *paPointsRes;

      MatrixElemAt (pT, 0, 0) = t * t * t;
      MatrixElemAt (pT, 0, 1) = t * t;
      MatrixElemAt (pT, 0, 2) = t;
      MatrixElemAt (pT, 0, 3) = 1;



      pResult = MatrixMultiply (pT, pMs);
      paPointsRes = MatrixMultiplyPoint (pResult, &paPoints [i], 4);

      if (i == 0 && t == 0.0)
        pDC->MoveTo(paPointsRes[0].x, paPointsRes[0].y);
      else
        pDC->LineTo(paPointsRes [0].x, paPointsRes [0].y);

      free (paPointsRes);
      MatrixDestroy (pResult);
    }
  }

  pDC->SelectObject(pOldPen);
//  DeleteObject(Stift);

  MatrixDestroy (pMs);
  MatrixDestroy (pT);
}

/*
 * Initializes the Spline matrix *pMs and allocates memory for *pT.
 * The caller is responsible for freeing the memory.
 */
BOOL CSplines::InitBetaSpline (PMATRIX *pMs, PMATRIX *pT, double dBeta1, double dBeta2)
{
  assert (pMs != NULL && pT != NULL);

  *pMs = MatrixCreate (4, 4);
  if (*pMs != NULL) {
    double k = dBeta2 + 2 * pow (dBeta1, 3) + 4 * pow (dBeta1, 2) +
               4 * dBeta1 + 2;

    MatrixElemAt (*pMs, 0, 0) = -2 * pow (dBeta1, 3) / k;
    MatrixElemAt (*pMs, 0, 1) = 2 * (dBeta2 + pow (dBeta1, 3) +
                                     pow (dBeta1, 2) + dBeta1) / k;
    MatrixElemAt (*pMs, 0, 2) = -2 * (dBeta2 + pow (dBeta1, 2) +
                                      dBeta1 + 1) / k;
    MatrixElemAt (*pMs, 0, 3) = 2 / k;

    MatrixElemAt (*pMs, 1, 0) = 6 * pow (dBeta1, 3) / k;
    MatrixElemAt (*pMs, 1, 1) = - 3 * (dBeta2 + 2 * pow (dBeta1, 3) +
                                     2 * pow (dBeta1, 2)) / k;
    MatrixElemAt (*pMs, 1, 2) = 3 * (dBeta2 + 2 * pow (dBeta1, 2)) / k;

    MatrixElemAt (*pMs, 2, 0) = -6 * pow (dBeta1, 3) / k;
    MatrixElemAt (*pMs, 2, 1) = 6 * (pow (dBeta1, 3) - dBeta1) / k;
    MatrixElemAt (*pMs, 2, 2) = 6 * dBeta1 / k;

    MatrixElemAt (*pMs, 3, 0) = 2 * pow (dBeta1, 3) / k;
    MatrixElemAt (*pMs, 3, 1) = (dBeta2 + 4 * (pow (dBeta1, 2) + dBeta1)) / k;
    MatrixElemAt (*pMs, 3, 2) = 2 / k;

    *pT = MatrixCreate (1, 4);
    if (*pT == NULL) {
      MatrixDestroy (*pMs);
      *pMs = NULL;
    }
  }

  return *pMs != NULL && *pT != NULL;
}


/*
 * Creates a matrix with nRows * nCols dimension
 * The matrix is initialized with all it's elements with 0
 */
PMATRIX CSplines::MatrixCreate (size_t nRows, size_t nCols)
{
	PMATRIX pMatrix;

	assert (nRows > 0 && nCols > 0);

  pMatrix = (PMATRIX) malloc (sizeof (MATRIX));
  if (pMatrix != NULL) {
    size_t i;

    pMatrix->nCols = nCols;
    pMatrix->nRows = nRows;
    pMatrix->ppElems = (double **) malloc (sizeof (double) * nRows);
    if (pMatrix->ppElems == NULL) {
      free (pMatrix);
      pMatrix = NULL;
    }

    for (i = 0; i < nRows; i++) {
      pMatrix->ppElems [i] = (double *) calloc (sizeof (double), nCols);
      if (pMatrix->ppElems [i]== NULL) {
       size_t k;

       for (k = 0; k < i; k++)
         free (pMatrix->ppElems [k]);

       free (pMatrix->ppElems);
       free (pMatrix);
       pMatrix = NULL;
       break;
      }
    }
  }

  return pMatrix;
}

/*
 * Frees the memory used by the matrix
 */
void CSplines::MatrixDestroy (PMATRIX pMatrix) 
{
  size_t i;

  assert (pMatrix != NULL);

  for (i = 0; i < pMatrix->nRows; i++)
    free (pMatrix->ppElems [i]);

  free (pMatrix->ppElems);
  free (pMatrix);
}

/*
 * Multiplies 2 matrixs.
 */
PMATRIX CSplines::MatrixMultiply (const PMATRIX pLeft, const PMATRIX pRight)
{
  PMATRIX pDest;
  size_t nLine;
  size_t nLeftRows, nLeftCols, nRightCols; 

  assert (pLeft != NULL && pRight != NULL && pLeft->nCols == pRight->nRows);

  pDest = MatrixCreate (pLeft->nRows, pRight->nCols);
  if (pDest != NULL) {
    nLeftCols = pLeft->nCols;
    nLeftRows = pLeft->nRows;
    nRightCols = pRight->nCols;

    for (nLine = 0; nLine < nLeftRows; nLine++) {
      size_t nCol;

      for (nCol = 0; nCol < nRightCols; nCol++) {
        size_t k;
        double temp;
      
        temp = 0;
        for (k = 0; k < nLeftCols; k++)
          temp += pLeft->ppElems [nLine][k] * pRight->ppElems [k][nCol];

        pDest->ppElems [nLine][nCol] = temp;
      }
    }
  }

  return pDest;
}

/*
 * Multiplies a matrix with a vector.
 */
POINT * CSplines::MatrixMultiplyPoint (const PMATRIX pMatrix, const POINT paPoint [], size_t nSize)
{
  POINT * paDest;
      
  assert (pMatrix != NULL && paPoint != NULL && pMatrix->nCols == nSize);


  paDest = (POINT *) malloc (sizeof (POINT) * nSize);
  if (paDest != NULL) {
    size_t nLine;

    for (nLine = 0; nLine < pMatrix->nRows; nLine++) {
      size_t i;
      double x, y;


      x = y = 0;
      for (i = 0; i < nSize; i++) {
		    x += pMatrix->ppElems [nLine][i] * paPoint [i].x;
		    y += pMatrix->ppElems [nLine][i] * paPoint [i].y;
	    }

      paDest [nLine].x = (long) x;
      paDest [nLine].y = (long) y;
    }
  }

  return paDest;
}
